#!/bin/sh

BACKUP_PATH=$2


# Prepare parameters
LANG="C"
LC_ALL="C"
export LANG
export LC_ALL


setup_db_env()
{
    IMSS_HOME=${IMSS_HOME:=/opt/trend/imss}
    ODBCINI=${IMSS_HOME}/config/odbc.ini
    ODBCSYSINI=${IMSS_HOME}/config
    DBINI=${IMSS_HOME}/config/database.ini
       
    DBSERVER=`grep "Servername" $ODBCINI | awk -F'=' '{print $2}'`
    DBUSER=`grep "UserId" $DBINI | awk -F= '{print $2}'`
    DBUSERPASS=`grep "Password" $DBINI | awk -F= '{print $2}'`
    PGPASSWORD="$DBUSERPASS"
    DBNAME=`grep "Database=" $DBINI | awk -F= '{print $2}'`
    
    
    DBSERVER=${DBSERVER:=127.0.0.1}
    DBUSER=${DBUSER:=sa}
    DBUSERPASS=${DBUSERPASS:=postgreSQL}
    DBNAME=${DBNAME:=imss}
    OLDPATH=${OLDPATH:=.}
    PSQL_CMD=${PSQL_CMD:=${IMSS_HOME}/PostgreSQL/bin/psql}
    PGDIR=${IMSS_HOME}/PostgreSQL/bin/
}



SUCCESS=0
SUCCESS_NOT_NEED_UPDATE=1
ERROR_NEED_ROLLBAK=2
ERROR_NO_ROLLBACK=3
ERROR_MUTLI_VIRUS_RULE=4
ERROR_ROLLBACK_EXCEPTION=5
ECHO=echo
ACTION=""

DisplayMsg(){
    if [ "$1" -eq "0" ] ; then
        $ECHO "$2"
        $ECHO `date`---"$2" >> $BACKUP_PATH/ImsaInstallSP1.log
    elif [ "$1" -eq "1" ] ; then
        $ECHO `date`---"$2" >> $BACKUP_PATH/ImsaInstallSP1.log
    fi
}

ExitWithLog()
{
	DisplayMsg "1" "apply_db.sh $ACTION exit with return code:$1"
	return $1
}


KeyExist()
{
	key=$1
	EXIST=1
	SQL="select count(*) from tb_global_setting where name='$1';"
	tmp=`echo $SQL|$PSQL_CMD -t -h $DBSERVER -U$DBUSER -d $DBNAME`
	if [ "$tmp" -eq 0 ];then
		EXIST=0
	fi
}


Backup(){
      $ECHO "IMSA 7.0 SP1 DB Backup is not supported!" >> $BACKUP_PATH/ImsaInstallSP1.log

}

GenerateNewSQL()
{
	[ -f $BACKUP_PATH/new_imss7.sql ] && rm -f $BACKUP_PATH/new_imss7.sql
	
	#DB table schema udpate
	NEWSQL1="ALTER TABLE t_cachedip ALTER column \"time\" type timestamp with time zone;"
 	NEWSQL2="ALTER TABLE t_cachedip_tmp ALTER column \"time\" type timestamp with time zone;"
 	NEWSQL3="ALTER TABLE t_cachedip_tmp2 ALTER column \"time\" type timestamp with time zone;"
	NEWSQL4="ALTER TABLE tb_euq_action ALTER column rcpt type varchar(2048);"
	NEWSQL5="ALTER TABLE tb_system_event DROP CONSTRAINT system_event_scanner_id_fk;"
	NEWSQL6="CREATE INDEX idx_euq_action_msgid_scannerid ON tb_euq_action(msg_id, scanner_id);"
	
	#insert new keys
	NEWSQL7="INSERT INTO tb_global_setting VALUES ('Report', 'dblog_purge_start_time', '4', 'imss.ini', NULL);"
	NEWSQL8="INSERT INTO tb_global_setting VALUES ('Report', 'dbtrafsum_purge_by_day', '62', 'imss.ini', NULL);"
	NEWSQL9="INSERT INTO tb_postfixconfig VALUES ('apply_setting', 0, 'yes');"
	NEWSQL10="INSERT INTO tb_postfixitemattribute(fieldname,format,\"check\",\"type\",postfixaction,objfile,keymaps) values('apply_setting','string',0,4,0,'','main.cf');"
	NEWSQL11="INSERT INTO tb_global_setting VALUES ('general', 'policy_evt_log_path', '/opt/trend/imss/log', 'imss.ini', NULL);"
	NEWSQL12="INSERT INTO tb_global_setting VALUES ('general', 'policy_evt_log_prefix', 'polevt.imss', 'imss.ini', NULL);"
	NEWSQL13="INSERT INTO tb_global_setting VALUES ('smtp','blockEmptyMailFrom','no','imss.ini',NULL);"
	
	#update existing keys
	NEWSQL14="UPDATE tb_global_setting set value = '10240' where section='General-Notification' and name='FreeDiskSize'and value='1024';"
	NEWSQL15="UPDATE tb_global_setting set value='The free disk space under %FOLDERPATH% on server %HOSTNAME% is %FREESPACE% MB, which is below the threshold.' 
			 where name='FreeDiskSpace_Message' and value='The free disk space under %FOLDERPATH% on server %HOSTNAME% is %FREESPACE%, which is below the threshold.';"
	NEWSQL16="UPDATE tb_global_setting set value='The free disk space under %FOLDERPATH% on server %HOSTNAME% is %FREESPACE% MB, which is below the threshold.' 
			 where name='FreeDiskSpace_SNMPMessage' and value='The free disk space under %FOLDERPATH% on server %HOSTNAME% is %FREESPACE%, which is below the threshold.';"
	NEWSQL17="UPDATE tb_global_setting set value='180' 
			 where name='recv_timeout_secs' and value='60';"
	NEWSQL18="UPDATE tb_global_setting set value='180' 
			 where name='send_timeout_secs' and value='30';"
	NEWSQL19="UPDATE tb_global_setting set value='log/debugFile.txt;log/errorFile.txt;log/catalina.out;log/apache_log;log/mod_jk.log;log/rpt_graph_log.txt;log/trace.log' 
			 where name='misc_file_rotate_prefix';"
	NEWSQL20="UPDATE tb_global_setting set value='0' 
			 where name='VSIExtractFileRatioLimit' and value='100';"
	NEWSQL21="UPDATE tb_postfixitemattribute set postfixaction='2' where fieldname='queue_directory' and format = 'path' and \"check\"='1' and \"type\"='4' and \"postfixaction\"='1';"
	
	[ -f $BACKUP_PATH/new_imss7.sql ] && rm -f $BACKUP_PATH/new_imss7.sql
	for sql in "$NEWSQL1 $NEWSQL2 $NEWSQL3 $NEWSQL4 $NEWSQL5 $NEWSQL6 $NEWSQL7 $NEWSQL8 $NEWSQL9 $NEWSQL10 $NEWSQL11 $NEWSQL12 $NEWSQL13 $NEWSQL14 $NEWSQL15 $NEWSQL16 $NEWSQL17 $NEWSQL18 $NEWSQL19 $NEWSQL20 $NEWSQL21" 
	do
		echo $sql >>$BACKUP_PATH/new_imss7.sql

	done
}


test ! -d $BACKUP_PATH && mkdir $BACKUP_PATH

HaveMoreThanOneVirusFilter()
{
	VIRUS_FILTER_NUMBER=0;
	tmp=`echo "select count(*) from tb_rule where has_virus_filter=1;"|$PSQL_CMD -h $DBSERVER -U$DBUSER -d $DBNAME|sed -n -e '3p'`	
#	if [ "$tmp" -eq 0 ];then
#		VIRUS_FILTER_NUMBER=1
#	fi
  if [ $? -eq 0 ];then
	  VIRUS_FILTER_NUMBER="$tmp"
  fi
}

VirusFilterIsValid()
{
	VIRUS_FILTER_VALID=1;
	tmp=`echo "select rule_value from tb_rule where has_virus_filter=1;"|$PSQL_CMD -h $DBSERVER -U$DBUSER -d $DBNAME|sed -n -e '3p'`	
	SENDER=`echo $tmp | awk -F";" '{print $5}'`
	RECIPIENT=`echo $tmp | awk -F";" '{print $11}'`
	
	if [ "$SENDER" != '*@*' -o "$RECIPIENT" != '*@*' ];then
		VIRUS_FILTER_VALID=0
	fi
	
}

IsDBServerRunning()
{
	DBSRVERUP=0
	$PSQL_CMD -h $DBSERVER -U$DBUSER -d $DBNAME >/dev/null 2>&1<<EOF
	select count(*) from tb_global_setting;
EOF
	if [ $? -eq 0 ];then
		DBSRVERUP=1
	fi
}

update_proc()
{
	SQLFILES=""
	for FILES in `ls ${IMSS_HOME}/sql/postgresql/install/*SP.sql` ; do
    		SQLFILES="$SQLFILES \i $FILES"
	done
	#echo $SQLFILES
	${PSQL_CMD} -h $DBSERVER -U $DBUSER -d $DBNAME >>$BACKUP_PATH/ImsaDbSP1.log 2>&1 << end_of_file
	$SQLFILES
end_of_file
}

setup_db_env
ACTION=$1
case $1 in
        "install")
        	IsDBServerRunning
        	if [ "$DBSRVERUP" -eq 0 ];then
        		DisplayMsg "1" "failed to connect to database"
        		ExitWithLog $ERROR_NO_ROLLBACK
        	fi
   	
		Backup
		GenerateNewSQL
		$PSQL_CMD -h $DBSERVER -U$DBUSER -d $DBNAME <$BACKUP_PATH/new_imss7.sql >>$BACKUP_PATH/ImsaDbSP1.log 2>&1
		update_proc
		if [ $? -ne 0 ];then
			DisplayMsg "1" "failed to update database"
			ExitWithLog $ERROR_NO_ROLLBACK
		fi
		ExitWithLog $SUCCESS
                 
        ;;
        "rollback")
        	IsDBServerRunning
        	if [ "$DBSRVERUP" -eq 0 ];then
        		DisplayMsg "1" "failed to connect to database"
        		ExitWithLog $ERROR_NO_ROLLBACK
        	fi
        	

        	HaveMoreThanOneVirusFilter
        	if [ "$VIRUS_FILTER_NUMBER" -ne 1 ];then
        		DisplayMsg "1" "None or multi-Virus rule exist"
        		ExitWithLog $ERROR_MUTLI_VIRUS_RULE
        	fi
        	
        	VirusFilterIsValid
        	if [ $VIRUS_FILTER_VALID -ne 1 ];then
        		DisplayMsg "1" "sender or recipient not euq to '*@*'"
        		ExitWithLog $ERROR_MUTLI_VIRUS_RULE
        	fi
        	
        	[ ! -f $BACKUP_PATH/backup_imss7.sql ] && echo "$BACKUP_PATH/backup_imss7.sql did not exist" && ExitWithLog $SUCCESS
			$PSQL_CMD -h $DBSERVER -U$DBUSER -d $DBNAME <$BACKUP_PATH/backup_imss7.sql >>$BACKUP_PATH/ImsaDbSP1.log 2>&1                            
			if [ $? -ne 0 ];then
		 		DisplayMsg "1" "failed to rollback database"
		 		ExitWithLog $ERROR_ROLLBACK_EXCEPTION
			fi
			
			[ -f $BACKUP_PATH/backup_imss7.sql ] && mv $BACKUP_PATH/backup_imss7.sql $BACKUP_PATH/rollbacked_imss7.sql

        ;;
        *)
			Usage $0
        ;;
esac

ExitWithLog 0


