//
// OBJECT: SelectList
//
// [DESC] SelectList object is a wrapper class for the existing Select object.
// [IN]   _list [Object]: Select object
//
function SelectList(_list)
{
    this.list = _list;
    this.add  = _add;
    this.deleteSelected = _deleteSelected;
    this.selectAll = _selectAll;
    this.clear = _clear;
    this.clean = _clean;
}

// [DESC] Add an item to the list.  text must contain a value.
//        If value is whitespace, value == text.
// [IN]   text [String]: Item's display value.
//        value [String]: Item's internal value.
// [OUT]  None.
function _add(text, value)
{
    if (text.isWhitespace())
        return;

    var newOption = (value.isWhitespace()) ? new Option(text,text) : 
                                             new Option(text,value);

    listLength = this.list.options.length;
    this.list.options[listLength] = newOption;
}

function _deleteSelected()
{
    for (var i = this.list.options.length - 1; i >= 0; i--)
    {
        if (this.list.options[i].selected)
            this.list.options[i] = null;
    }
}

function _selectAll()
{
    for (var i = this.list.options.length - 1; i >= 0; i--)
        this.list.options[i].selected = true;
}

function _clear()
{
    for (var i = this.list.options.length - 1; i >= 0; i--)
        this.list.options[i] = null;
}

function _clean()
{
    for (var i = this.list.options.length - 1; i >= 0; i--)
    {
       if (this.list.options[i].text.isWhitespace())
           this.list.options[i] = null;
    }
}
