#!/bin/sh

usage()
{
	echo "$0 <start|stop|status|reload|restart>"
	exit
}

IMSS_HOME=/opt/trend/imss
PG_BASE=$IMSS_HOME/PostgreSQL
PG_BIN_DIR=$PG_BASE/bin
PG_LIB_DIR=$PG_BASE/lib
PGDATA=/var/imss/pgdata
PG_LOG_FILE=/var/imss/pglog
PG_PID_FILE=$PGDATA/postmaster.pid
PG_LOCK_FILE="/tmp/.s.PGSQL.5432.lock"

PS_CMD="ps -ef --width 1000"

PATH=$PG_BIN_DIR:$PATH
LD_LIBRARY_PATH=$PG_LIB_DIR:$LD_LIBRARY_PATH
export PATH LD_LIBRARY_PATH
export PGDATA

trap "" SIGINT
trap "unlock" EXIT

. $IMSS_HOME/script/imssfunctions

lock

delete_lock_file()
{
	if [ -f $PG_LOCK_FILE ]; then
		echo "lock file $PG_LOCK_FILE already exists"
		rm -f $PG_LOCK_FILE > /dev/null 2>&1
		echo "rm -f $PG_LOCK_FILE"
	fi
}

start()
{
	if [ -f $PG_PID_FILE ]; then
		PID=`cat $PG_PID_FILE | head -n 1`
		DB_EXIST=`$PS_CMD | grep $PID | grep -w postmaster | grep -v grep`
		
		if test "$DB_EXIST"; then
			echo "DB is running"
		else
			echo "DB is not running"
			rm -f $PG_PID_FILE > /dev/null 2>&1
			echo "rm -f $PG_PID_FILE"

			delete_lock_file
		fi
	else
		delete_lock_file
	fi

	$IMSS_HOME/bin/close_handle_sh "su - imss -c \"$PG_BIN_DIR/pg_ctl -D '$PGDATA' -w -o \\\"-i\\\" -l $PG_LOG_FILE start\" </dev/null"
}

stop()
{
	$IMSS_HOME/bin/close_handle_sh "su - imss -c \"$PG_BIN_DIR/pg_ctl -D '$PGDATA' -w -m fast stop\""

	rm -f $PG_PID_FILE  > /dev/null 2>&1
	rm -f $PG_LOCK_FILE > /dev/null 2>&1
}
case $1 in
	start)
		start
		;;
	stop)
		stop
		rm $PGDATA/postmaster.pid  > /dev/null 2>&1
		;;
	status)
		su - imss -c "$PG_BIN_DIR/pg_ctl -D '$PGDATA' status"
		;;
	reload)
		su - imss -c "$PG_BIN_DIR/pg_ctl -D '$PGDATA' reload"
		;;
	restart)
#		su - imss -c "$PG_BIN_DIR/pg_ctl -D '$PGDATA' -w -m fast -o \"-i\" -l $PG_LOG_FILE restart"
		stop
		start
		;;
	*)
		usage
		;;
esac

