#! /bin/bash

PATH=/sbin:/usr/sbin:/bin:/usr/bin:$PATH
export PATH

IMSS_HOME=/opt/trend/imss
LD_LIBRARY_PATH=$IMSS_HOME/lib
export IMSS_HOME
export LD_LIBRARY_PATH
ODBCINI=$IMSS_HOME/config/odbc.ini
export ODBCINI
ODBCSYSINI=$IMSS_HOME/config
export ODBCSYSINI

RTTRAFFIC=$IMSS_HOME/bin/rt_mail_traffic
PID_FILE=$IMSS_HOME/bin/rt_mail_traffic.pid
PS_CMD="ps -ef --width 1000"

trap "" SIGINT
trap "unlock" EXIT

. $IMSS_HOME/script/imssfunctions

lock

start() {
	for i in `$PS_CMD | grep -w rt_mail_traffic | grep -v grep | grep -v defunct | grep -v log | grep -v $0 | awk '{ print $2 }'` ; do
		if test "$i" ; then
			echo "$RTTRAFFIC is running."
			exit 1
		fi
	done
	if test -x $RTTRAFFIC ; then
		$RTTRAFFIC
		for i in `$PS_CMD | grep -w rt_mail_traffic | grep -v grep | grep -v log | grep -v $0 | awk '{ print $2 }'`; do
			if test "$i" ; then
				echo "$RTTRAFFIC has started."
				exit 1
			fi
		done
		echo "$RTTRAFFIC cannot be started."
	else
		echo "$RTTRAFFIC is not executable."
		exit 1
	fi
}

stop() {
	if [ -f $PID_FILE ]; then
		PID=`cat $PID_FILE`
	fi	
	if [ "$PID" != "" ]; then
		EXIST=`$PS_CMD | grep $PID | grep -w rt_mail_traffic | grep -v grep | grep -v log | grep -v $0`
		if test "$EXIST" ; then
			echo "Shutting down rt_mail_traffic daemon $PID ..."
			kill -TERM $PID
			COUNT=0
			while $PS_CMD | grep -w rt_mail_traffic | grep -v grep | grep -v log | grep -v $0 > /dev/null 2>&1 ; do
				sleep 1
				COUNT=`expr $COUNT + 1`
				if test "$COUNT" = "30" ; then
					for i in `$PS_CMD | grep -w rt_mail_traffic | grep -v grep | grep -v log | grep -v $0 | awk '{ print $2 }'` ; do
						echo "Force to kill $i ..."
						kill -KILL $i
					done
					break
				fi
			done
		else
			echo "rt_mail_traffic daemon $PID is not running."
		fi
	else
		echo "$PID_FILE missing"
		for i in `$PS_CMD | grep -w rt_mail_traffic | grep -v grep | grep -v log | grep -v $0 | awk '{ print $2 }'` ; do
			if test "$i" ; then
				kill -KILL $i
			fi
		done
	fi
	rm -f $PID_FILE >/dev/null 2>&1
}

reload() {
	if [ -f $PID_FILE ]; then
		PID=`cat $PID_FILE`
		ps -p $PID > /dev/null
		if [ $? -eq 0 ] ; then
			echo "reload rt_mail_traffic ..."
			kill -s SIGHUP $PID
		else
			echo "rt_mail_traffic daemon $PID is not running."
		fi
	else
		echo "$PID_FILE missing."
	fi
}

case "$1" in
'start')
	start
	;;

'stop')
	stop
	;;

'restart')
	stop
	start
	;;

'reload')
	reload
	;;

'?')
	ps -ef | grep -w rt_mail_traffic | grep -v grep | grep -v log | grep -v $0 > /dev/null
	if [ $? -eq 0 ] ; then
		echo "yes"
	else
		echo "no"
	fi
	;;

*)
	echo "usage: $0 start|stop|reload|restart"
	exit 1
	;;
esac
