#!/bin/sh

DATABASE_NAME="$3"
USER_NAME="$1"
HOST_NAME="$2"
USER_PASS="$4"
PG_BASE=/opt/trend/imss
PG_BIN=$PG_BASE/PostgreSQL/bin
#PG_LIB=$PG_BASE/PostgreSQL/lib/postgresql
PATH=$PG_BIN:$PATH ; export PATH

usage()
{
	echo "Usage: $0 <USER_NAME> <HOST_NAME> <DATABASE_NAME>"
	exit
}

#if [ $# != 0 ] && [ $# != 3 ] ; then
#	usage
#fi
#if USER_NAME is null on solaris,test -z $USER_NAME will fail,
#so we have to modify it
if [ -z "$USER_NAME" ] ; then
	USER_NAME=sa
fi
if [ -z "$HOST_NAME" ] ; then
	HOST_NAME=localhost
fi
if [ -z "$DATABASE_NAME" ] ; then
	DATABASE_NAME=imss
fi
if [ ! -z "$USER_PASS" ] ; then
	PGPASSWORD="$USER_PASS"
	export PGPASSWORD
fi
#echo $DATABASE_NAME
#echo $USER_NAME
#echo $HOST_NAME

# Define global initialization sql scripts.
# these scripts will be invoked (in the same sequence as specified)
# before any other sql scripts
#

while :
do
psql -h $HOST_NAME -U $USER_NAME -d template1 << end_of_file > hold1
select datname from pg_stat_activity where datname='$DATABASE_NAME';
end_of_file
grep $DATABASE_NAME hold1 > /dev/null
if [ $? = 0 ] ; then
    rm -f hold1
    echo ""
    echo "$DATABASE_NAME database is currently being accessed by other users, please first disconnect them before install database."
    echo "Do you want to continue to install ?"
    echo ""
    echo "    [Y|y] = retry install  [N|n] = quit install"
    echo ""
    printf "Please type a letter (default is yes) [     ]\b\b\b\b\b"
    read ans
    case ${ans} in
      [nN]|[nN]o|NO ) exit 1;;
                  * ) ;;
    esac
else
    rm -f hold1
    break
fi
done
dropdb -h $HOST_NAME -U $USER_NAME $DATABASE_NAME 2>/dev/null
createdb -h $HOST_NAME -U $USER_NAME -E unicode $DATABASE_NAME
createlang -h $HOST_NAME -U $USER_NAME -d $DATABASE_NAME plpgsql

GLOBAL_SQLFILES="\i create.sql"
IMSA_SQL_SCHEMA="\i create_imsa.sql"
IMSA_SQL_DATA="\i insert_imsa.sql"
INSERT_SQLFILE="\i insert.sql"
INSERT_KEYWORD="\i insertKeywords.sql"
SQLFILES=""
for FILES in `ls *SP.sql` ; do
	SQLFILES="$SQLFILES \i $FILES"
done
#echo $GLOBAL_SQLFILES
#echo $SQLFILES

psql -h $HOST_NAME -U $USER_NAME -d $DATABASE_NAME >/dev/null << end_of_file
$GLOBAL_SQLFILES
end_of_file

if [ -f /etc/trend-release ]; then
psql -h $HOST_NAME -U $USER_NAME -d $DATABASE_NAME >/dev/null << end_of_file
$IMSA_SQL_SCHEMA
end_of_file
fi

psql -h $HOST_NAME -U $USER_NAME -d $DATABASE_NAME >/dev/null << end_of_file
$SQLFILES
end_of_file

psql -h $HOST_NAME -U $USER_NAME -d $DATABASE_NAME >/dev/null << end_of_file
$INSERT_SQLFILE
end_of_file

if [ -f /etc/trend-release ]; then
psql -h $HOST_NAME -U $USER_NAME -d $DATABASE_NAME >/dev/null << end_of_file
$IMSA_SQL_DATA
end_of_file
fi

psql -h $HOST_NAME -U $USER_NAME -d $DATABASE_NAME >/dev/null << end_of_file
$INSERT_KEYWORD
end_of_file
