
SET client_encoding = 'LATIN9';
SET check_function_bodies = false;
SET client_min_messages = warning;

CREATE or REPLACE FUNCTION ipint2string(bigint) RETURNS character varying
    AS 'declare
	IPv4 alias for $1;
	binIPv4 int8;
	ret varchar(20);
BEGIN 
ret:='''';
binIPv4:=IPv4;
ret:=ret||(binIPv4>>24)||''.'';
binIPv4:=binIPv4&16777215;
ret:=ret||(binIPv4>>16)||''.'';
binIPv4:=binIPv4&65535;
ret:=ret||(binIPv4>>8)||''.'';
binIPv4:=binIPv4&255;
ret:=ret||(binIPv4>>0);
return ret;
END;
'
    LANGUAGE plpgsql;


CREATE TABLE t_messagesdropped (
    upstreammta_ip bigint DEFAULT 0,
    recordtime timestamp with time zone,
    "type" integer,
    count integer
)
WITHOUT OIDS;


--
-- add index in t_messagesdropped
-- jerry IMSA
--
CREATE INDEX idx_messagesdropped_recordtime ON t_messagesdropped USING btree (recordtime);

CREATE TABLE t_iprule (
    "enable" boolean NOT NULL,
    "domain" character varying(256),
    ip bigint NOT NULL,
    "type" smallint NOT NULL,
    "action" smallint NOT NULL,
    applyflag boolean DEFAULT false NOT NULL,
    createtime timestamp without time zone NOT NULL,
    comments varchar(256)
)
WITHOUT OIDS;


CREATE VIEW t_blockedip AS
    SELECT t_iprule."domain", t_messagesdropped.upstreammta_ip AS ip, t_messagesdropped.recordtime AS createtime, t_messagesdropped."type", t_messagesdropped.count FROM (t_messagesdropped LEFT JOIN t_iprule ON ((t_messagesdropped.upstreammta_ip = t_iprule.ip)));


COMMENT ON VIEW t_blockedip IS 'select t_iprule.domain,t_messagesdropped.upstreammta_ip as ip,t_messagesdropped.recordtime as createtime,t_messagesdropped.Type ,t_messagesdropped.count from t_messagesdropped left join t_iprule on t_messagesdropped.upstreammta_ip=t_iprule.ip';


CREATE TABLE t_cachedip (
    ip bigint NOT NULL,
    isspam smallint DEFAULT 0 NOT NULL,
    isdha smallint DEFAULT 0 NOT NULL,
    isvirus smallint DEFAULT 0 NOT NULL,
    isbounced smallint DEFAULT 0 NOT NULL,
    isnullsession smallint DEFAULT 0 NOT NULL,
    isnormal smallint DEFAULT 0 NOT NULL,
    "time" timestamp with time zone,
    processflag boolean DEFAULT false,
    totalmsg smallint DEFAULT 0
)
WITHOUT OIDS;


CREATE TABLE t_cachedip_tmp (
    ip bigint,
    isspam smallint,
    isdha smallint,
    isvirus smallint,
    isbounced smallint,
    isnullsession smallint,
    isnormal smallint,
    "time" timestamp with time zone,
    processflag boolean,
    totalmsg smallint DEFAULT 0
)
WITHOUT OIDS;


--
-- Name: t_manual_domain; Type: TABLE; Schema: public;  Tablespace: 
--

CREATE TABLE t_manual_domain (
    "enable" boolean NOT NULL,
    "domain" character varying(256) NOT NULL,
    ip bigint,
    "type" smallint NOT NULL,
    "action" smallint NOT NULL,
    createtime timestamp without time zone NOT NULL,
    applyflag boolean DEFAULT false NOT NULL,
    comments character varying(256)
)
WITHOUT OIDS;


CREATE TABLE t_messages (
    msgindex serial NOT NULL,
    msgid character varying(256),
    upstreammta_ip bigint,
    msgiscompleted boolean,
    msgisspam boolean,
    msgcontainvirus boolean,
    sessionstarttime timestamp with time zone,
    msgstarttime timestamp with time zone,
    c_mailfrom_returncode integer,
    c_mailfrom_mailbox character varying(64),
    c_mailfrom_domain character varying(256),
    c_mailfrom_domainip bigint,
    msgisbounced boolean,
    total_rcptto_count integer,
    noexist_rcptto_count integer,
    unknown_rcptto_count integer,
    subject character varying(256),
    isautosubmit boolean
)
WITHOUT OIDS;


CREATE TABLE t_messagesnull (
    msgindex serial NOT NULL,
    upstreammta_ip bigint NOT NULL,
    sessionstarttime timestamp with time zone
)
WITHOUT OIDS;


CREATE TABLE t_type_setting (
    "type" smallint NOT NULL,
    "enable" boolean NOT NULL,
    duration integer NOT NULL,
    totalmail integer NOT NULL,
    rate smallint NOT NULL,
    recipients integer DEFAULT 0 NOT NULL,
    nonexistrcpt integer DEFAULT 0 NOT NULL,
    "action" smallint NOT NULL,
    delayduration integer NOT NULL DEFAULT 0
)
WITHOUT OIDS;


CREATE TABLE t_type_setting_default (
    "type" smallint NOT NULL,
    "enable" boolean NOT NULL,
    "Duration" integer NOT NULL,
    totalmail integer NOT NULL,
    rate smallint NOT NULL,
    recipients integer NOT NULL,
    "action" smallint NOT NULL,
    nonexistrcpt integer,
    delayduration integer NOT NULL DEFAULT 0
)
WITHOUT OIDS;



CREATE TABLE t_foxhuntersetting
(
  "type" int2 NOT NULL,
  "enable" bool NOT NULL DEFAULT false,
  CONSTRAINT t_foxhuntersetting_pkey PRIMARY KEY ("type")
)
WITHOUT OIDS;




-- Name: tb_active_update; Type: TABLE; Schema: public;  Tablespace: 
--

CREATE TABLE tb_active_update (
    component_id serial NOT NULL,
    "type" integer NOT NULL,
    content oid NOT NULL,
    update_time timestamp with time zone NOT NULL,
    version character varying(32),
    file_size integer NOT NULL,
    file_name character varying(128) NOT NULL,
    "valid" bit(1) DEFAULT B'0'::"bit",
    apply_fail integer NOT NULL
)
WITHOUT OIDS;



CREATE SEQUENCE "tb_administrator_admin_id_seq"
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;



-- Sequence: tb_scanner_id_seq
CREATE SEQUENCE "tb_scanner_id_seq"
  INCREMENT 1
  MINVALUE 1
  MAXVALUE 9223372036854775807
  START 1
  CACHE 1;

--
-- Name: tb_administrator; Type: TABLE; Schema: public;  Tablespace: 
--

CREATE TABLE tb_administrator (
    admin_id integer DEFAULT nextval('tb_administrator_admin_id_seq') NOT NULL,
    admin_name character(256) NOT NULL,
    enabled boolean DEFAULT true NOT NULL,
    using_imss_auth integer DEFAULT 1 NOT NULL,
    own_root_entity character(256),
    md5_digest character(64),
    summary_permission integer DEFAULT 0 NOT NULL,
    policy_permission integer DEFAULT 0 NOT NULL,
    ipfiltering_permission integer DEFAULT 0 NOT NULL,
    reports_permission integer DEFAULT 0 NOT NULL,
    logs_permission integer DEFAULT 0 NOT NULL,
    quarantines_permission integer DEFAULT 0 NOT NULL,
    system_permission integer DEFAULT 0 NOT NULL
)
WITHOUT OIDS;


--
-- Name: tb_cmagent_lock; Type: TABLE; Schema: public;  Tablespace: 
--

CREATE TABLE tb_cmagent_lock (
    redalertid character varying(255),
    issuetime  character varying(255)
)
WITHOUT OIDS;

CREATE SEQUENCE "tb_cmagent_version_id_seq"
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

--
-- Name: tb_component_list; Type: TABLE; Schema: public;  Tablespace: 
--

CREATE TABLE tb_component_list (
    scanner_id integer DEFAULT nextval('tb_scanner_id_seq'),
    scanner_name character varying(1024),
    ip_addr character varying(256),
    daemon smallint DEFAULT 0,
    policy smallint DEFAULT 0,
    euq smallint DEFAULT 0,
    nrs smallint DEFAULT 0,
    ipprofiler smallint DEFAULT 0,
    euq_port int4 DEFAULT 0,
    admin_cmd int4 DEFAULT 0
)
WITHOUT OIDS;


--
-- Name: tb_entity; Type: TABLE; Schema: public;  Tablespace: 
--

CREATE TABLE tb_entity (
    entity_id serial NOT NULL,
    entity_type integer NOT NULL,
    entity_name character varying(321) NOT NULL,
    root_entity smallint DEFAULT 0::smallint
)
WITHOUT OIDS;


--
-- Name: tb_entity_rule; Type: TABLE; Schema: public;  Tablespace:
--

CREATE TABLE tb_entity_rule (
    entity_id integer NOT NULL,
    rule_id integer NOT NULL
)
WITHOUT OIDS;


--
-- Name: tb_euq_db_info; Type: TABLE; Schema: public;  Tablespace: 
--

CREATE TABLE tb_euq_db_info (
    server character varying(256),
    port integer,
    username character varying(256),
    "password" character varying(256),
    db_name character varying(256),
    db_id serial NOT NULL,
    db_type character varying(32),
    status int2 DEFAULT 1
)
WITHOUT OIDS;


--
-- Name: tb_euq_entity; Type: TABLE; Schema: public;  Tablespace: 
--

CREATE TABLE tb_euq_entity (
    entity_id serial NOT NULL,
    entity_name character varying(256)
)
WITHOUT OIDS;


--
-- Name: tb_euq_table_info; Type: TABLE; Schema: public;  Tablespace: 
--

CREATE TABLE tb_euq_table_info (
    table_id integer NOT NULL,
    db_id integer NOT NULL
)
WITHOUT OIDS;


--
-- Name: tb_euq_whitelist; Type: TABLE; Schema: public;  Tablespace: 
--

CREATE TABLE tb_euq_action
(
  msg_id char(36) NOT NULL,
  scanner_id int4 NOT NULL,
  rcpt varchar(2048) NOT NULL,
  "action" int4 NOT NULL,
  subject varchar(256),
  msg_size float8,
  area_id varchar(256),
  time_date timestamptz,
  sender varchar(256)
)
WITHOUT OIDS;

--
-- Name: tb_global_setting; Type: TABLE; Schema: public;  Tablespace: 
--

CREATE TABLE tb_global_setting (
    section character varying(256) NOT NULL,
    name character varying(256) NOT NULL,
    value character varying(2048),
    inifile character varying(256) NOT NULL,
    notes character(16)
)
WITHOUT OIDS;


--
-- Name: tb_inter_addr; Type: TABLE; Schema: public;  Tablespace: 
--

CREATE TABLE tb_inter_addr (
    "DomainName" character varying(256),
    "DomainType" integer DEFAULT 0,
    "DomainID" integer DEFAULT 0
)
WITHOUT OIDS;


--
-- Name: tb_ip_event; Type: TABLE; Schema: public;  Tablespace: 
--

CREATE TABLE tb_ip_event (
    time_date timestamp with time zone NOT NULL,
    scanner_id integer NOT NULL,
    ip_address character varying(64),
    "action" integer NOT NULL,
    act_num integer NOT NULL
)
WITHOUT OIDS;

--
-- add index in tb_ip_event
-- jerry, IMSA team
--

CREATE INDEX idx_ip_event_time_date ON tb_ip_event USING btree (time_date);


--
-- Name: tb_mda_event; Type: TABLE; Schema: public;  Tablespace: 
--

CREATE TABLE tb_mda_event (
    date_time timestamp with time zone,
    msg_id character(36),
    sender character varying(321),
    recipients character varying(2048),
    "action" character varying(128)
)
WITHOUT OIDS;


--
-- Name: tb_named_obj; Type: TABLE; Schema: public;  Tablespace: 
--

CREATE TABLE tb_named_obj (
    id serial NOT NULL,
    "type" integer NOT NULL,
    name character varying(1024) NOT NULL,
    content text,
    msg_count integer DEFAULT 0,
    msg_size float8 DEFAULT 0
)
WITHOUT OIDS;


--
-- Name: tb_named_obj_rule; Type: TABLE; Schema: public;  Tablespace: 
--

CREATE TABLE tb_named_obj_rule (
    id integer NOT NULL,
    rule_id integer NOT NULL
)
WITHOUT OIDS;


--
-- Name: tb_named_obj_scan_exception; Type: TABLE; Schema: public;  Tablespace: 
--

CREATE TABLE tb_named_obj_scan_exception (
    id integer NOT NULL,
    "type" integer NOT NULL
)
WITHOUT OIDS;


--
-- Name: tb_policy; Type: TABLE; Schema: public;  Tablespace: 
--

CREATE TABLE tb_policy (
    policy_id serial NOT NULL,
    policy_name character varying(256),
    policy_type integer NOT NULL,
    is_enable integer NOT NULL,
    is_default integer NOT NULL,
    is_hidden integer NOT NULL,
    create_by character varying(256) NOT NULL,
    modify_by character varying(256) NOT NULL,
    creation_time timestamp with time zone,
    last_modified_time timestamp with time zone
)
WITHOUT OIDS;



--
-- Name: tb_splited_virus_event; Type: TABLE; Schema: public;  Tablespace: 
--

CREATE TABLE tb_splited_virus_event (
    virus_event_id serial NOT NULL,
    time_date timestamp with time zone NOT NULL,
    virus_name character varying(256),
    virus_type character varying(80)
)
WITHOUT OIDS;


--
-- Name: tb_policy_type; Type: TABLE; Schema: public;  Tablespace: 
--

CREATE TABLE tb_policy_type (
    policy_type integer,
    type_name character varying(30) NOT NULL,
    time_to_live integer NOT NULL,
    update_version integer,
    note text
)
WITHOUT OIDS;

--
-- Name: tb_top10_individual_traffic; Type: TABLE; Schema: public;  Tablespace: 
--

CREATE TABLE tb_top10_individual_traffic (
    date timestamp with time zone NOT NULL,
    user_name character varying(256),
    "domain" character varying(256),
    spam_no numeric DEFAULT 0::numeric,
    virus_no numeric DEFAULT 0::numeric,
    total_no numeric DEFAULT 0::numeric,
    spam_size double precision DEFAULT 0::double precision,
    virus_size double precision DEFAULT 0::double precision,
    total_size double precision DEFAULT 0::double precision,
    rcpt_or_sender bit(1) DEFAULT B'0'::"bit"
)
WITHOUT OIDS;


--
-- Name: tb_report_results; Type: TABLE; Schema: public;  Tablespace: 
--

CREATE TABLE tb_report_results (
    report_id serial NOT NULL,
    ticket_id integer NOT NULL,
    create_time timestamp with time zone,
    schedule_report_name character varying(256),
    admin_id integer,
    report_type integer,
    request_timestamp timestamp with time zone,
    report_filepath character varying(256) NOT NULL,
    report_csvpath character varying(256) NOT NULL
)
WITHOUT OIDS;


--
-- Name: tb_report_setting; Type: TABLE; Schema: public;  Tablespace: 
--

CREATE TABLE tb_report_setting (
    report_type integer,
    admin_id integer,
    reports_save integer
)
WITHOUT OIDS;


--
-- Name: tb_report_tickets; Type: TABLE; Schema: public;  Tablespace: 
--

CREATE TABLE tb_report_tickets (
    ticket_id serial NOT NULL,
    admin_id integer,
    report_type integer,
    request_timestamp timestamp with time zone,
    report_name character varying(256),
    report_status integer DEFAULT 0,
    scanner_name character varying(1024) DEFAULT 'all'::character varying,
    report_item character varying(256),
    report_start_day integer,
    reset_column boolean,
    time_start timestamp with time zone,
    time_end timestamp with time zone,
    run_today smallint DEFAULT 0,
    isenabled boolean
)
WITHOUT OIDS;


--
-- Name: tb_report_type; Type: TABLE; Schema: public;  Tablespace: 
--

CREATE TABLE tb_report_type (
    report_type_id serial NOT NULL,
    report_type_desc character varying(50)
)
WITHOUT OIDS;


--
-- Name: tb_rule; Type: TABLE; Schema: public;  Tablespace: 
--

CREATE TABLE tb_rule (
    rule_id serial NOT NULL,
    policy_id integer NOT NULL,
    version_number integer NOT NULL,
    rule_name character varying(1024) NOT NULL,
    active_time integer DEFAULT 247 NOT NULL,
    rule_type integer NOT NULL,
    display_action integer NOT NULL,
    has_multi_actions integer NOT NULL,
    has_virus_filter integer NOT NULL,
    has_spam_filter integer NOT NULL,
    has_attachment_filter integer NOT NULL,
    has_content_filter integer NOT NULL,
    has_size_filter integer NOT NULL,
    has_time_range_filter integer NOT NULL,
    has_other_filter integer NOT NULL,
    note character varying(4096),
    rule_value text NOT NULL,
    rule_order integer NOT NULL
)
WITHOUT OIDS;


--
-- Name: tb_scanning_exceptions; Type: TABLE; Schema: public;  Tablespace: 
--

CREATE TABLE tb_scanning_exceptions (
    "type" integer NOT NULL,
    display_actions character varying(256),
    value character varying(2048)
)
WITHOUT OIDS;



--
-- Name: tb_system_event; Type: TABLE; Schema: public;  Tablespace: 
--

CREATE TABLE tb_system_event (
    system_event_id serial NOT NULL,
    scanner_id integer NOT NULL,
    time_date timestamp with time zone NOT NULL,
    event_id integer DEFAULT 0,
    event_description character varying(256)
)
WITHOUT OIDS;


--
-- Name: tb_traffic_summary; Type: TABLE; Schema: public;  Tablespace: 
--

CREATE TABLE tb_traffic_summary
(
  starttime timestamptz NOT NULL,
  endtime timestamptz NOT NULL,
  msg_total numeric DEFAULT (0)::numeric,
  msg_in numeric DEFAULT (0)::numeric,
  msg_out numeric DEFAULT (0)::numeric,
  msg_total_size float8 DEFAULT (0)::double precision,
  msg_in_size float8 DEFAULT (0)::double precision,
  msg_out_size float8 DEFAULT (0)::double precision,
  pol_virus_total numeric DEFAULT (0)::numeric,
  pol_virus_in numeric DEFAULT (0)::numeric,
  pol_virus_out numeric DEFAULT (0)::numeric,  
  pol_spyware_total numeric DEFAULT (0)::numeric,
  pol_spyware_in numeric DEFAULT (0)::numeric,
  pol_spyware_out numeric DEFAULT (0)::numeric,
  pol_spam_total numeric DEFAULT (0)::numeric,
  pol_spam_in numeric DEFAULT (0)::numeric,
  pol_spam_out numeric DEFAULT (0)::numeric,
  pol_phish_total numeric DEFAULT (0)::numeric,
  pol_phish_in numeric DEFAULT (0)::numeric,
  pol_phish_out numeric DEFAULT (0)::numeric,
  pol_attach_total numeric DEFAULT (0)::numeric,
  pol_attach_in numeric DEFAULT (0)::numeric,
  pol_attach_out numeric DEFAULT (0)::numeric,
  pol_size_total numeric DEFAULT (0)::numeric,
  pol_size_in numeric DEFAULT (0)::numeric,
  pol_size_out numeric DEFAULT (0)::numeric,
  pol_content_total numeric DEFAULT (0)::numeric,
  pol_content_in numeric DEFAULT (0)::numeric,
  pol_content_out numeric DEFAULT (0)::numeric,
  pol_other_total numeric DEFAULT (0)::numeric,
  pol_other_in numeric DEFAULT (0)::numeric,
  pol_other_out numeric DEFAULT (0)::numeric,
  pol_exception_total numeric DEFAULT (0)::numeric,
  pol_exception_in numeric DEFAULT (0)::numeric,
  pol_exception_out numeric DEFAULT (0)::numeric,
  pol_ipfilt_total numeric DEFAULT (0)::numeric,
  pol_ipfilt_in numeric DEFAULT (0)::numeric,
  pol_ipfilt_out numeric DEFAULT (0)::numeric,
  pol_nrs_total numeric DEFAULT (0)::numeric,
  pol_nrs_in numeric DEFAULT (0)::numeric,
  pol_nrs_out numeric DEFAULT (0)::numeric,
  ipfilt_conn numeric DEFAULT (0)::numeric,
  ipfilt_block numeric DEFAULT (0)::numeric,
  ipfilt_virus numeric DEFAULT (0)::numeric,
  ipfilt_spam numeric DEFAULT (0)::numeric,
  ipfilt_bounce numeric DEFAULT (0)::numeric,
  ipfilt_dha numeric DEFAULT (0)::numeric,
  ipfilt_manual numeric DEFAULT (0)::numeric,
  nrs_conn numeric DEFAULT (0)::numeric,
  nrs_block numeric DEFAULT (0)::numeric,
  spam_quar numeric DEFAULT (0)::numeric,
  spam_del numeric DEFAULT (0)::numeric,
  spam_subtag numeric DEFAULT (0)::numeric,
  spam_other numeric DEFAULT (0)::numeric,
  spam_total_size float8 DEFAULT (0)::double precision,
  spam_quar_size float8 DEFAULT (0)::double precision,
  spam_del_size float8 DEFAULT (0)::double precision,
  spam_subtag_size float8 DEFAULT (0)::double precision,
  spam_other_size float8 DEFAULT (0)::double precision,
  virus_del numeric DEFAULT (0)::numeric,
  virus_quar numeric DEFAULT (0)::numeric,
  virus_clean numeric DEFAULT (0)::numeric,
  virus_attdel numeric DEFAULT (0)::numeric
) 
WITHOUT OIDS;


--
-- Name: tb_version_number; Type: TABLE; Schema: public;  Tablespace: 
--

CREATE TABLE tb_version_number (
    global_settings integer DEFAULT 0 NOT NULL,
    named_objects integer DEFAULT 0 NOT NULL,
    policies integer DEFAULT 0 NOT NULL,
    manual_order integer DEFAULT 0 NOT NULL,
    scan_exceptions integer DEFAULT 0 NOT NULL,
    au_uploads integer DEFAULT 0 NOT NULL,
    internal_address integer DEFAULT 0 NOT NULL
)
WITHOUT OIDS;


--
-- Name: tb_postfixconfig; Type: TABLE; Schema: public;  Tablespace: 
--

CREATE TABLE tb_postfixconfig (
    fieldname character varying(256) NOT NULL,
    version integer DEFAULT 0 NOT NULL,
    fieldvalue text
)
WITHOUT OIDS;


--
-- Name: tb_postfixitemattribute; Type: TABLE; Schema: public;  Tablespace: 
--

CREATE TABLE tb_postfixitemattribute
(
  fieldname character varying(256) NOT NULL,
  format character varying(64) NOT NULL,
  "check" integer NOT NULL,
  "type" integer NOT NULL,
  postfixaction integer NOT NULL,
  objfile character varying(256) NOT NULL,
  keymaps character varying(256) NOT NULL
)
WITHOUT OIDS;
 

--
-- Name: AdminId; Type: CONSTRAINT; Schema: public;  Tablespace: 
--

ALTER TABLE ONLY tb_administrator
    ADD CONSTRAINT "admin_id" PRIMARY KEY ("admin_id");


--
-- Name: FilterID; Type: CONSTRAINT; Schema: public;  Tablespace: 
--

--
-- Name: PK_T_MESSAGES; Type: CONSTRAINT; Schema: public;  Tablespace: 
--

ALTER TABLE ONLY t_messages
    ADD CONSTRAINT "PK_T_MESSAGES" PRIMARY KEY (msgindex);


--
-- Name: PK_T_MESSAGESNULL; Type: CONSTRAINT; Schema: public;  Tablespace: 
--

ALTER TABLE ONLY t_messagesnull
    ADD CONSTRAINT "PK_T_MESSAGESNULL" PRIMARY KEY (msgindex);


--
-- Name: areaid; Type: CONSTRAINT; Schema: public;  Tablespace: 
--

--
-- Name: au_component_id_pk; Type: CONSTRAINT; Schema: public;  Tablespace: 
--

ALTER TABLE ONLY tb_active_update
    ADD CONSTRAINT au_component_id_pk PRIMARY KEY (component_id);


--
-- Name: au_type_unique; Type: CONSTRAINT; Schema: public;  Tablespace: 
--

ALTER TABLE ONLY tb_active_update
    ADD CONSTRAINT au_type_unique UNIQUE ("type");


--
-- Name: component_list_scanner_id_pk; Type: CONSTRAINT; Schema: public; Tablespace: 
--

ALTER TABLE ONLY tb_component_list
    ADD CONSTRAINT component_list_scanner_id_pk PRIMARY KEY (scanner_id);


--
-- Name: namedobj_id_pk; Type: CONSTRAINT; Schema: public;  Tablespace: 
--

ALTER TABLE ONLY tb_named_obj
    ADD CONSTRAINT namedobj_id_pk PRIMARY KEY (id);


--
-- Name: pk_msg_action; Type: CONSTRAINT; Schema: public;  Tablespace: 
--


--
-- Name: pk_tb_euq_db_info; Type: CONSTRAINT; Schema: public;  Tablespace: 
--

ALTER TABLE ONLY tb_euq_db_info
    ADD CONSTRAINT pk_tb_euq_db_info PRIMARY KEY (db_id);

--
-- Name: pk_tb_euq_table_info; Type: CONSTRAINT; Schema: public;  Tablespace: 
--

ALTER TABLE ONLY tb_euq_table_info
    ADD CONSTRAINT pk_tb_euq_table_info PRIMARY KEY (table_id, db_id);


--
-- Name: pk_tb_global_setting; Type: CONSTRAINT; Schema: public;  Tablespace: 
--

ALTER TABLE ONLY tb_global_setting
    ADD CONSTRAINT pk_tb_global_setting PRIMARY KEY (section, name, inifile);


--
-- Name: policy_id; Type: CONSTRAINT; Schema: public;  Tablespace: 
--

ALTER TABLE ONLY tb_policy
    ADD CONSTRAINT policy_id PRIMARY KEY (policy_id);


--
-- Name: prioritylist_entityid_policyid_pk; Type: CONSTRAINT; Schema: public;  Tablespace: 
--


--
-- Name: report_results_report_id_pk; Type: CONSTRAINT; Schema: public;  Tablespace: 
--

ALTER TABLE ONLY tb_report_results
    ADD CONSTRAINT report_results_report_id_pk PRIMARY KEY (report_id);


--
-- Name: report_tickets_admin_id_type_request_unique; Type: CONSTRAINT; Schema: public;  Tablespace: 
--

ALTER TABLE ONLY tb_report_tickets
    ADD CONSTRAINT report_tickets_admin_id_type_request_unique UNIQUE (admin_id, report_type, request_timestamp);


--
-- Name: report_tickets_ticket_id_pk; Type: CONSTRAINT; Schema: public;  Tablespace: 
--

ALTER TABLE ONLY tb_report_tickets
    ADD CONSTRAINT report_tickets_ticket_id_pk PRIMARY KEY (ticket_id);


--
-- Name: report_type_id_pk; Type: CONSTRAINT; Schema: public;  Tablespace: 
--

ALTER TABLE ONLY tb_report_type
    ADD CONSTRAINT report_type_id_pk PRIMARY KEY (report_type_id);


--
-- Name: tb_report_setting_report_type_key; Type: CONSTRAINT; Schema: public;  Tablespace: 
--

ALTER TABLE ONLY tb_report_setting
    ADD CONSTRAINT tb_report_setting_report_type_key UNIQUE (report_type);
    

--
-- Name: rule_id; Type: CONSTRAINT; Schema: public;  Tablespace: 
--

ALTER TABLE ONLY tb_rule
    ADD CONSTRAINT rule_id PRIMARY KEY (rule_id);


--
-- Name: scanning_exceptions_pk; Type: CONSTRAINT; Schema: public;  Tablespace: 
--

ALTER TABLE ONLY tb_scanning_exceptions
    ADD CONSTRAINT scanning_exceptions_pk PRIMARY KEY ("type");


--
-- Name: system_event_system_event_id_pk; Type: CONSTRAINT; Schema: public;  Tablespace: 
--

ALTER TABLE ONLY tb_system_event
    ADD CONSTRAINT system_event_system_event_id_pk PRIMARY KEY (system_event_id);


--
-- Name: t_iprule_pkey; Type: CONSTRAINT; Schema: public;  Tablespace: 
--

ALTER TABLE ONLY t_iprule
    ADD CONSTRAINT t_iprule_pkey PRIMARY KEY (ip);


--
-- Name: t_manual_domain_pkey; Type: CONSTRAINT; Schema: public;  Tablespace: 
--

ALTER TABLE ONLY t_manual_domain
    ADD CONSTRAINT t_manual_domain_pkey PRIMARY KEY ("domain");


--
-- Name: t_type_setting_default_pkey; Type: CONSTRAINT; Schema: public;  Tablespace: 
--

ALTER TABLE ONLY t_type_setting_default
    ADD CONSTRAINT t_type_setting_default_pkey PRIMARY KEY ("type");


--
-- Name: t_type_setting_pkey; Type: CONSTRAINT; Schema: public;  Tablespace: 
--

ALTER TABLE ONLY t_type_setting
    ADD CONSTRAINT t_type_setting_pkey PRIMARY KEY ("type");


--
-- Name: tb_entity_entity_name_key; Type: CONSTRAINT; Schema: public;  Tablespace: 
--

ALTER TABLE ONLY tb_entity
    ADD CONSTRAINT tb_entity_entity_name_key UNIQUE (entity_name);


--
-- Name: tb_top10_individual_traffic_pkey; Type: CONSTRAINT; Schema: public;  Tablespace: 
--

ALTER TABLE ONLY tb_top10_individual_traffic
    ADD CONSTRAINT tb_top10_individual_traffic_pkey PRIMARY KEY (date, user_name, "domain", rcpt_or_sender);


--
-- Name: tb_traffic_summary_pkey; Type: CONSTRAINT; Schema: public;  Tablespace: 
--

ALTER TABLE ONLY tb_traffic_summary
    ADD CONSTRAINT tb_traffic_summary_pkey PRIMARY KEY (starttime, endtime);


--
-- Name: tb_cmagent_lock_index; Type: INDEX; Schema: public;  Tablespace: 
--

CREATE UNIQUE INDEX tb_cmagent_lock_index ON tb_cmagent_lock USING btree (redalertid, issuetime);




ALTER TABLE ONLY tb_named_obj_rule
    ADD CONSTRAINT id FOREIGN KEY (id) REFERENCES tb_named_obj(id) ON UPDATE RESTRICT ON DELETE RESTRICT;


ALTER TABLE ONLY tb_named_obj_scan_exception
    ADD CONSTRAINT id FOREIGN KEY (id) REFERENCES tb_named_obj(id) ON UPDATE RESTRICT ON DELETE RESTRICT;

ALTER TABLE ONLY tb_named_obj_scan_exception
    ADD CONSTRAINT "type" FOREIGN KEY ("type") REFERENCES tb_scanning_exceptions("type") ON UPDATE RESTRICT ON DELETE RESTRICT;


ALTER TABLE ONLY tb_rule
    ADD CONSTRAINT policy_id FOREIGN KEY (policy_id) REFERENCES tb_policy(policy_id) ON UPDATE CASCADE ON DELETE CASCADE;


ALTER TABLE ONLY tb_report_results
    ADD CONSTRAINT report_results_ticket_id_fk FOREIGN KEY (ticket_id) REFERENCES tb_report_tickets(ticket_id) ON DELETE CASCADE;


ALTER TABLE ONLY tb_report_setting
    ADD CONSTRAINT report_setting_report_type_fk FOREIGN KEY (report_type) REFERENCES tb_report_type(report_type_id);


ALTER TABLE ONLY tb_report_tickets
    ADD CONSTRAINT report_tickets_report_type_fk FOREIGN KEY (report_type) REFERENCES tb_report_type(report_type_id);


ALTER TABLE ONLY tb_named_obj_rule
    ADD CONSTRAINT rule_id FOREIGN KEY (rule_id) REFERENCES tb_rule(rule_id) ON UPDATE RESTRICT ON DELETE RESTRICT;


CREATE INDEX idx_euq_action_msgid_scannerid ON tb_euq_action(msg_id, scanner_id);
CREATE INDEX idx_system_event_scannerid ON tb_system_event(scanner_id);
CREATE INDEX idx_system_event_timedate ON tb_system_event(time_date);
CREATE INDEX idx_splited_virus_event_timedate ON tb_splited_virus_event(time_date);
CREATE INDEX idx_report_results_ticketid ON tb_report_results(ticket_id);
CREATE INDEX idx_report_results_schedule ON tb_report_results(schedule_report_name);
CREATE INDEX idx_report_tickets_type ON tb_report_tickets(report_type);
CREATE INDEX idx_report_tickets_adminid_type ON tb_report_tickets(admin_id, report_type);
CREATE INDEX idx_report_tickets_type_runtoday ON tb_report_tickets(report_type, run_today);
CREATE INDEX idx_report_tickets_type_time1 ON tb_report_tickets(report_type, time_start, run_today);
CREATE INDEX idx_report_tickets_type_time2 ON tb_report_tickets(report_type, report_start_day, time_start, run_today);

