-- This is the db schema definition for IMSA

-- add a column (is_master) in tb_comonent_list
-- to indicate if the box is Master or Slave
ALTER TABLE tb_component_list ADD COLUMN is_master int DEFAULT 0;

-- for real-time statistics
-- cpu usage
CREATE TABLE tb_rtstat_cpu
(
    scanner_id  int NOT NULL,
    rt_time     timestamptz,
    user_usage  float DEFAULT (0)::double precision,
    sys_usage   float DEFAULT (0)::double precision,
    idle_usage  float DEFAULT (0)::double precision,
    constraint  cpu_scanner_id_pk primary key (scanner_id)
);

-- memory usage
CREATE TABLE tb_rtstat_mem
(
    scanner_id  int NOT NULL,
    rt_time     timestamptz,
    mem_total   int DEFAULT 0,
    mem_used    int DEFAULT 0,
    mem_avail   int DEFAULT 0,
    constraint  mem_scanner_id_pk primary key (scanner_id)
);

-- disk usage
CREATE TABLE tb_rtstat_disk_size
(
    scanner_id  int NOT NULL,
    type        int NOT NULL,
    rt_time     timestamptz,
    disk_total  int DEFAULT 0,
    disk_used   int DEFAULT 0,
    disk_avail  int DEFAULT 0,
    constraint  disk_scanner_id_type_pk primary key (scanner_id, type)
);

-- mail queue count
CREATE TABLE tb_rtstat_mta_queue_count
(
    scanner_id  int NOT NULL,
    type        int NOT NULL,
    rt_time     timestamptz,
    mail_count  int DEFAULT 0,
    constraint  queue_scanner_id_type_pk primary key (scanner_id, type)
);

-- mail traffic of IMSS
CREATE TABLE tb_rtstat_mail_traffic_policy
(
    scanner_id      int NOT NULL,
    interval        int NOT NULL,
    starttime       timestamptz NOT NULL,
    endtime         timestamptz NOT NULL,
    msg_total       numeric DEFAULT (0)::numeric,
    msg_inbound     numeric DEFAULT (0)::numeric,
    msg_outbound    numeric DEFAULT (0)::numeric,
    pol_normal      numeric DEFAULT (0)::numeric,
    pol_virus       numeric DEFAULT (0)::numeric,
    pol_spyware     numeric DEFAULT (0)::numeric,
    pol_spam        numeric DEFAULT (0)::numeric,
    pol_phish       numeric DEFAULT (0)::numeric,
    pol_attach      numeric DEFAULT (0)::numeric,
    pol_size        numeric DEFAULT (0)::numeric,
    pol_content     numeric DEFAULT (0)::numeric,
    pol_other       numeric DEFAULT (0)::numeric,
    pol_exception   numeric DEFAULT (0)::numeric,
    CONSTRAINT      tb_rtstat_mail_traffic_policy_pkey PRIMARY KEY (scanner_id, interval, starttime, endtime)
);
CREATE INDEX idx_mail_traffic_policy_endtime
    ON tb_rtstat_mail_traffic_policy
    USING btree
    (endtime);

-- traffic of IP filtering
CREATE TABLE tb_rtstat_mail_traffic_ipfilt
(
    scanner_id      int NOT NULL,
    interval        int NOT NULL,
    starttime       timestamptz NOT NULL,
    endtime         timestamptz NOT NULL,
    ipfilt_total    numeric DEFAULT (0)::numeric,
    ipfilt_virus    numeric DEFAULT (0)::numeric,
    ipfilt_spam     numeric DEFAULT (0)::numeric,
    ipfilt_bounce   numeric DEFAULT (0)::numeric,
    ipfilt_dha      numeric DEFAULT (0)::numeric,
    ipfilt_manual   numeric DEFAULT (0)::numeric,
    ipfilt_nrs      numeric DEFAULT (0)::numeric,
    CONSTRAINT      tb_rtstat_mail_traffic_ipfilt_pkey PRIMARY KEY (scanner_id, interval, starttime, endtime)
);
CREATE INDEX idx_mail_traffic_ipfilt_endtime
    ON tb_rtstat_mail_traffic_ipfilt
    USING btree
    (endtime);

-- realtime statistic settings
CREATE TABLE tb_rtstat_setting
(
  sys_refresh int4 DEFAULT 60,
  traffic_refresh int4 DEFAULT 300,
  cpu_threshold int2 DEFAULT 50,
  mem_threshold int4 DEFAULT 50
);

-- add scanner_id for foxhunter log
-- since we need to analyse foxhunter flow for each scanner
ALTER TABLE t_messagesdropped ADD COLUMN scanner_id int DEFAULT 0;

-- add scanner_id for view t_blockedip
DROP VIEW t_blockedip;
CREATE VIEW t_blockedip AS
    SELECT t_iprule."domain", t_messagesdropped.scanner_id AS scanner_id, t_messagesdropped.upstreammta_ip AS ip, t_messagesdropped.recordtime AS createtime, t_messagesdropped."type", t_messagesdropped.count FROM (t_messagesdropped LEFT JOIN t_iprule ON ((t_messagesdropped.upstreammta_ip = t_iprule.ip)));

ALTER TABLE public.t_blockedip OWNER TO sa;

COMMENT ON VIEW t_blockedip IS 'select t_iprule.domain,t_messagesdropped.scanner_id as scanner_id,t_messagesdropped.upstreammta_ip as ip,t_messagesdropped.recordtime as createtime,t_messagesdropped.Type ,t_messagesdropped.count from t_messagesdropped left join t_iprule on t_messagesdropped.upstreammta_ip=t_iprule.ip';

-- add deferred management tables
CREATE TABLE tb_deferred
(
  msg_id char(36) NOT NULL,
  scanner_id int4 NOT NULL,
  smtp_sender varchar(321),
  smtp_rcpt varchar(2048),
  arrive_time timestamptz,
  defer_reason varchar(4096),
  msg_size int4 DEFAULT 0,
  status int2 DEFAULT 0,
  CONSTRAINT tb_deferred_pk PRIMARY KEY (msg_id, scanner_id)
);
CREATE TABLE tb_deferred_action
(
  msg_id char(36) NOT NULL,
  scanner_id int4 NOT NULL,
  "action" int2 NOT NULL
);
CREATE INDEX idx_tb_deferred_msgid_sid
  ON tb_deferred
  USING btree
  (msg_id, scanner_id);

CREATE TABLE tb_postpone

(
  msg_id char(36) NOT NULL,
  scanner_id int4 NOT NULL,
  qa_id int4 NOT NULL,
  smtp_sender varchar(321),
  smtp_rcpt varchar(2048),
  msg_subject varchar(512),
  msg_route int2,
  msg_size float8 DEFAULT 0,
  rule_name varchar(256),
  time_date timestamptz,
  filter_type_virus bit(1) DEFAULT B'0'::"bit",
  filter_type_spyware bit(1) DEFAULT B'0'::"bit",
  filter_type_spam bit(1) DEFAULT B'0'::"bit",
  filter_type_attachment bit(1) DEFAULT B'0'::"bit",
  filter_type_content bit(1) DEFAULT B'0'::"bit",
  filter_type_size bit(1) DEFAULT B'0'::"bit",
  filter_type_other bit(1) DEFAULT B'0'::"bit",
  filter_type_malformed bit(1) DEFAULT B'0'::"bit",
  filter_type_unscanned bit(1) DEFAULT B'0'::"bit",
  filter_type_phishing bit(1) DEFAULT B'0'::"bit",
  attach_name varchar(1024),
  qt_live_ttl timestamptz,
  ref_num int4 DEFAULT 0
)
WITHOUT OIDS;

-- For patch via web
CREATE TABLE tb_patch_latest_pkg
(
  file_name varchar(128),
  patch_type int4,
  apply_order int4,
  build_number varchar(128),
  patch_title varchar(1024),
  upload_time timestamptz,
  timeout int4,
  patch_description varchar(256),
  patch_interruption varchar(256),
  eula_content oid,
  file_content oid
)
WITHOUT OIDS;
ALTER TABLE tb_patch_latest_pkg OWNER TO sa;

CREATE TABLE tb_patch_status
(
  scanner_id int4,
  is_master int4,
  ip_addr varchar(256),
  scanner_name varchar(1024),
  status int4,
  paction int4,
  result int4,
  error_msg varchar(1024)
)
WITHOUT OIDS;
ALTER TABLE tb_patch_status OWNER TO sa;

CREATE TABLE tb_patch_history
(
  scanner_id int4,
  rollbackable int4,
  applied_date timestamptz,
  patch_type int4,
  build_number varchar(128),
  patch_title varchar(1024),
  timeout int4
) 
WITHOUT OIDS;
ALTER TABLE tb_patch_history OWNER TO sa;

ALTER TABLE tb_administrator ADD COLUMN cli_permission int4 NOT NULL DEFAULT 0;

ALTER TABLE tb_component_list ADD COLUMN os_ver varchar(128);
ALTER TABLE tb_component_list ADD COLUMN app_ver varchar(128);
ALTER TABLE tb_component_list ADD COLUMN patch_log oid;
ALTER TABLE tb_component_list ADD COLUMN mac_addr varchar(128);

CREATE TABLE tb_trusted_ip_list
(
  ip_addr varchar NOT NULL,
  scanner_id int4 NOT NULL DEFAULT -1,
  CONSTRAINT trust_ip_list_pk PRIMARY KEY (ip_addr)
);

alter table tb_version_number add column system_settings int4 NOT NULL DEFAULT 0;

CREATE TABLE tb_cdt_detail
(
    scanner_id  int NOT NULL,
    file_days   int NOT NULL,
    file_time   timestamptz,
    file_size   int ,
    file_stat   int ,
    file_name   varchar(128),
    file_oid    oid,
    file_etime  timestamptz
);
