#!/bin/sh

SYS=`uname -s`
if test $SYS = "SunOS" ; then
  MACHINE="Sol"
else if test $SYS = "Linux" ; then 
  MACHINE="Linux"
fi
fi

die()
{
	echo $@ >&2
	exit 1
}

BASE_DIR=$1
IMSS_DIR=$2
DBPASS=$3
PG_LOCALE=$4
PG_BASE=$BASE_DIR/PostgreSQL
PG_DATA_DIR=/var/imss
PG_BIN=$PG_BASE/bin
PG_LIB=$PG_BASE/lib
PGDATA=$PG_DATA_DIR/pgdata
PGSHARE=$PG_BASE/share/postgresql
CURDIR=$IMSS_DIR/script
chmod +x $CURDIR/dbctl.sh

# Stop postmaster if it's running.
if [ -x $CURDIR/dbctl.sh -a -x $PG_BIN/pg_ctl ]; then
	$CURDIR/dbctl.sh stop > /dev/null 2>&1
fi

#PATH=$PG_BIN:$PATH
#LD_LIBRARY_PATH=$PG_LIB:$LD_LIBRARY_PATH
#export PATH LD_LIBRARY_PATH
#export PGDATA
#export PATH
PATH=/sbin:$PATH
export PATH

rm -rf /var/imss
mkdir -p $PG_DATA_DIR || die "Failed to create $PG_DATA_DIR"
mkdir -m 0700 -p $PGDATA
touch $PG_DATA_DIR/pg_err.log
chown -R imss $PG_DATA_DIR

# Prepare data directory.
echo "Set up PostgreSQL database"

sed -e "s,\$libdir,$PG_LIB/postgresql,g" $PGSHARE/conversion_create.sql > $BASE_DIR/temp/pp$$
cp $BASE_DIR/temp/pp$$ $PGSHARE/conversion_create.sql      
rm $BASE_DIR/temp/pp$$

su - imss -c "$PG_BIN/initdb -L $PGSHARE -D $PGDATA --locale=$PG_LOCALE" > /dev/null 2>>$PG_DATA_DIR/pg_err.log || die "Failed to initialize database"

#in case of not allowing tcp/ip connection
if [ -f /etc/trend-release ]; then
	sed -e "s,#tcpip_socket =.*,tcpip_socket = true,g" -e "s,max_connections =.*,max_connections = 600,g" -e "s,shared_buffers =.*,shared_buffers = 2000,g" $PGDATA/postgresql.conf > $BASE_DIR/temp/pp$$
else	
	sed -e "s,#tcpip_socket =.*,tcpip_socket = true,g" -e "s,max_connections =.*,max_connections = 300,g" $PGDATA/postgresql.conf > $BASE_DIR/temp/pp$$
fi

cp $BASE_DIR/temp/pp$$ $PGDATA/postgresql.conf      
rm $BASE_DIR/temp/pp$$

if [ -f /etc/trend-release ]; then
    IPADDR=`ifconfig -a|grep -v "127.0.0.1"|grep inet|awk -F' ' '{print $2}'|awk -F: '{print $2}' |head -1`
    NETMASK=`ifconfig -a|grep -v "127.0.0.1"|grep Mask | awk -F: '{print $4}'|head -1`
else if test $MACHINE = "Linux" ; then
    IPADDR=$REALADDR
    NETMASK=`ifconfig -a | grep "$REALADDR" | awk -F: '{print $4}'`
else if test $SYS = "SunOS" ; then 
    IPADDR=$REALADDR
    NETMASK=`ifconfig -a | grep "$REALADDR" | awk -F: '{print $4}'`
    fi
fi
fi

if [ -f /etc/trend-release ]; then
echo "host    all         all         0/0         password" >> $PGDATA/pg_hba.conf
else
echo "host    all         all         $IPADDR      $NETMASK   password" >> $PGDATA/pg_hba.conf
fi

# Start-up postmaster ( PostgreSQL process )
$CURDIR/dbctl.sh start

# Create user "sa".  Parameters are "no create new user" "allow create database" and "ask password"
DB_NAME=imss
echo "Creating user \"sa\""
# make sure system link to correct library
if [ ! -f /etc/trend-release ]; then
/sbin/ldconfig $PG_LIB > /dev/null 2>>$PG_DATA_DIR/pg_err.log || die "Failed to ldconfig $PG_LIB"
fi
RETRY_COUNTER=5
while true ; do
	if su - imss -c "$PG_BIN/createuser -a -d sa" ; then
		# Success
		break
	fi
	RETRY_COUNTER=`expr $RETRY_COUNTER - 1`
	if [ $RETRY_COUNTER -le 0 ] ; then
		echo "Failed to create user.  Set up database later by yourself."
		echo "Press Enter to continue ......"
		read TMP
		die "Failed to create user"
	fi
done
#su - imss -c "$PG_BIN/createdb -U sa -E UTF-8 $DB_NAME"
#RETRY_COUNTER=5
#while true ; do
#	if su - imss -c "$PG_BIN/createdb -U sa -E UTF-8 $DB_NAME" ; then
		# Success
#		break
#	fi
#	RETRY_COUNTER=`expr $RETRY_COUNTER - 1`
#	if [ $RETRY_COUNTER -le 0 ] ; then
#		echo "Failed to create database $DB_NAME.  Set up database later by yourself."
#		echo "Press Enter to continue ......"
#		read TMP
#		die "Failed to create database"
#	fi
#done
#su - imss -c "$PG_BIN/createlang plpgsql $DB_NAME"
#RETRY_COUNTER=5
#while true ; do
#	if su - imss -c "$PG_BIN/createlang plpgsql $DB_NAME" ; then
		# Success
#		break
#	fi
#	RETRY_COUNTER=`expr $RETRY_COUNTER - 1`
#	if [ $RETRY_COUNTER -le 0 ] ; then
#		echo "Failed to create language.  Set up database later by yourself."
#		echo "Press Enter to continue ......"
#		read TMP
#		die "Failed to create language"
#	fi
#done
# Create tables and stored procedures.
rm -f $PG_DATA_DIR/pg_err.log
su - imss -c "$PG_BIN/psql template1 -c \"ALTER USER sa WITH PASSWORD '$DBPASS'\""
PATH=$PG_BIN:$PATH ; export PATH
cd $IMSS_DIR/sql/postgresql/install ; ./InitDatabase.sh sa localhost

echo "PostgreSQL setup complete."

