CREATE TABLE tb_log_table_info
(
  table_id serial NOT NULL,
  hour_num int4 NOT NULL DEFAULT -1,
  time_start timestamptz,
  time_end timestamptz,
  top100data_bookmark int4 NOT NULL DEFAULT 0,
  trafficsum_bookmark int4 NOT NULL DEFAULT 0,
  reportsum_bookmark int4 NOT NULL DEFAULT 0,
  indexed bit(1) DEFAULT B'0'::"bit",
  summarized bit(1) DEFAULT B'0'::"bit",
  top100data_gened bit(1) DEFAULT B'0'::"bit",
  trafficsum_gened bit(1) DEFAULT B'0'::"bit",
  CONSTRAINT pk_tb_log_table_info PRIMARY KEY (table_id)
)
WITHOUT OIDS;

CREATE OR REPLACE FUNCTION CreateLogTables (int, int) RETURNS int AS '
DECLARE
    nFirstHour     ALIAS FOR $1;
    nCreateNum     ALIAS FOR $2;
    nTmp           int;
    nHourSeq       int;
    szCreateTableCmd text;
BEGIN
    SET client_min_messages to ERROR;
    nTmp := 0;
    while nCreateNum > nTmp loop
    nHourSeq := nFirstHour + nTmp;
    szCreateTableCmd := ''CREATE TABLE tb_policy_event_'' || nHourSeq || '' (
  policy_event_id serial NOT NULL,
  msg_id char(36) NOT NULL,
  scanner_id int4 NOT NULL,  
  smtp_sender varchar(321),
  smtp_rcpt varchar(2048),
  msg_subject varchar(256),
  msg_route int2,
  msg_size float8,
  rule_name varchar(256),
  time_date timestamptz NOT NULL,
  filter_type_virus bit(1) DEFAULT B''''0''''::"bit",
  filter_type_spyware bit(1) DEFAULT B''''0''''::"bit",
  filter_type_spam bit(1) DEFAULT B''''0''''::"bit",
  filter_type_attachment bit(1) DEFAULT B''''0''''::"bit",
  filter_type_content bit(1) DEFAULT B''''0''''::"bit",
  filter_type_size bit(1) DEFAULT B''''0''''::"bit",
  filter_type_other bit(1) DEFAULT B''''0''''::"bit",
  filter_type_malformed bit(1) DEFAULT B''''0''''::"bit",
  filter_type_unscanned bit(1) DEFAULT B''''0''''::"bit",
  filter_type_phishing bit(1) DEFAULT B''''0''''::"bit",
  filter_description varchar(256),
  act_deleted bit(1) DEFAULT B''''0''''::"bit",
  act_quarantined bit(1) DEFAULT B''''0''''::"bit",
  act_delivered bit(1) DEFAULT B''''0''''::"bit",
  act_archived bit(1) DEFAULT B''''0''''::"bit",
  act_rcptchanged bit(1) DEFAULT B''''0''''::"bit",
  act_stamped bit(1) DEFAULT B''''0''''::"bit",
  act_attachments_deleted bit(1) DEFAULT B''''0''''::"bit",
  act_subjects_tagged bit(1) DEFAULT B''''0''''::"bit",
  act_delivery_postponed bit(1) DEFAULT B''''0''''::"bit",
  act_notifications_sent bit(1) DEFAULT B''''0''''::"bit",
  act_cleaned bit(1) DEFAULT B''''0''''::"bit",
  act_bcc bit(1) DEFAULT B''''0''''::"bit",
  act_handoff bit(1) DEFAULT B''''0''''::"bit",
  spam_score float4 DEFAULT (0)::real,
  spam_category_id int4 DEFAULT 1,
  spam_sensitive int2 DEFAULT 0,
  attach_name varchar(1024),
  attach_type varchar(80),
  virus_name varchar(1024),
  virus_type varchar(80),
  original_msg_id varchar(1024),
  CONSTRAINT policy_event_'' || nHourSeq || ''_policy_event_id_pk PRIMARY KEY (policy_event_id)
) 
WITHOUT OIDS;'';
    EXECUTE szCreateTableCmd;

    szCreateTableCmd := ''CREATE TABLE tb_quarantine_'' || nHourSeq || '' (
  policy_event_id int4 NOT NULL,
  msg_id char(36) NOT NULL,
  scanner_id int4 NOT NULL,
  smtp_sender varchar(321),
  smtp_rcpt varchar(2048),
  msg_subject varchar(256),
  msg_route int2,
  msg_size float8,
  qa_id int4 NOT NULL,
  rule_name varchar(256),
  time_date timestamptz,
  filter_type_virus bit(1) DEFAULT B''''0''''::"bit",
  filter_type_spyware bit(1) DEFAULT B''''0''''::"bit",
  filter_type_spam bit(1) DEFAULT B''''0''''::"bit",
  filter_type_attachment bit(1) DEFAULT B''''0''''::"bit",
  filter_type_content bit(1) DEFAULT B''''0''''::"bit",
  filter_type_size bit(1) DEFAULT B''''0''''::"bit",
  filter_type_other bit(1) DEFAULT B''''0''''::"bit",
  filter_type_malformed bit(1) DEFAULT B''''0''''::"bit",
  filter_type_unscanned bit(1) DEFAULT B''''0''''::"bit",
  filter_type_phishing bit(1) DEFAULT B''''0''''::"bit",
  attach_name varchar(1024),
  qt_live_ttl timestamptz,
  ref_num int4 DEFAULT 0,
  original_msg_id varchar(1024),
  CONSTRAINT quarantine_'' || nHourSeq || ''_policy_event_id_pk PRIMARY KEY (policy_event_id)
) 
WITHOUT OIDS;'';
    EXECUTE szCreateTableCmd;

    szCreateTableCmd := ''CREATE TABLE tb_archive_'' || nHourSeq || '' (
  policy_event_id int4 NOT NULL,
  msg_id char(36) NOT NULL,
  scanner_id int4 NOT NULL,
  smtp_sender varchar(321),
  smtp_rcpt varchar(2048),
  msg_subject varchar(256),
  msg_route int2,
  msg_size float8,
  qa_id int4 NOT NULL,
  rule_name varchar(256),
  time_date timestamptz,
  filter_type_virus bit(1) DEFAULT B''''0''''::"bit",
  filter_type_spyware bit(1) DEFAULT B''''0''''::"bit",
  filter_type_spam bit(1) DEFAULT B''''0''''::"bit",
  filter_type_attachment bit(1) DEFAULT B''''0''''::"bit",
  filter_type_content bit(1) DEFAULT B''''0''''::"bit",
  filter_type_size bit(1) DEFAULT B''''0''''::"bit",
  filter_type_other bit(1) DEFAULT B''''0''''::"bit",
  filter_type_malformed bit(1) DEFAULT B''''0''''::"bit",
  filter_type_unscanned bit(1) DEFAULT B''''0''''::"bit",
  filter_type_phishing bit(1) DEFAULT B''''0''''::"bit",
  attach_name varchar(1024),
  qt_live_ttl timestamptz,
  original_msg_id varchar(1024),
  CONSTRAINT archive_'' || nHourSeq || ''_policy_event_id_pk PRIMARY KEY (policy_event_id)
) 
WITHOUT OIDS;'';
    EXECUTE szCreateTableCmd;
    szCreateTableCmd := ''CREATE INDEX idx_policy_event_'' || nHourSeq || ''_timedate ON tb_policy_event_'' || nHourSeq || ''(time_date)'';
    EXECUTE szCreateTableCmd;
    
    szCreateTableCmd := ''CREATE INDEX idx_quarantine_'' || nHourSeq || ''_timedate ON tb_quarantine_'' || nHourSeq || ''(time_date)'';
    EXECUTE szCreateTableCmd;
    
    szCreateTableCmd := ''CREATE INDEX idx_archive_'' || nHourSeq || ''_timedate ON tb_archive_'' || nHourSeq || ''(time_date)'';
    EXECUTE szCreateTableCmd;

    insert into tb_log_table_info (table_id, hour_num) values(nHourSeq, -1);
    nTmp := nTmp + 1;
    end loop;

    return 0;
END;
' LANGUAGE plpgsql;

select CreateLogTables(0, 192);

CREATE INDEX idx_log_table_info_hournum ON tb_log_table_info(hour_num);
CREATE INDEX idx_log_table_info_time_start ON tb_log_table_info(time_start);

CREATE OR REPLACE FUNCTION CreateLogImportFunc (int, int) RETURNS int AS '
DECLARE
    nFirstHour     ALIAS FOR $1;
    nCreateNum     ALIAS FOR $2;
    nTmp           int;
    nHourSeq       int;
    szCreateTableCmd text;
BEGIN
    nTmp := 0;
    while nCreateNum > nTmp loop
    nHourSeq := nFirstHour + nTmp;
    szCreateTableCmd := ''CREATE OR REPLACE FUNCTION AddPolicyEventFromLog_'' || nHourSeq || '' (
    char(36), int, varchar(321), varchar(2048), varchar(256), smallint, float,
    varchar(256), timestamptz, char(10), varchar(256), char(13), real, int, smallint, varchar(1024),
    varchar(80), varchar(1024),varchar(80), int, timestamptz, int, timestamptz, varchar(1024), bit, int)
RETURNS integer AS ''''
DECLARE
    v_msg_id              ALIAS FOR $1;
    v_scanner_id          ALIAS FOR $2;
    v_smtp_sender         ALIAS FOR $3;
    v_smtp_rcpt           ALIAS FOR $4;
    v_msg_subject         ALIAS FOR $5;
    v_msg_route           ALIAS FOR $6;
    v_msg_size            ALIAS FOR $7;
    v_rule_name           ALIAS FOR $8;
    v_time_date           ALIAS FOR $9;
    v_filter_types        ALIAS FOR $10;
    v_filter_description  ALIAS FOR $11;
    v_msg_actions         ALIAS FOR $12;
    v_spam_score          ALIAS FOR $13;
    v_spam_category_id    ALIAS FOR $14;
    v_spam_sensitive      ALIAS FOR $15;
    v_attach_name         ALIAS FOR $16;
    v_attach_type         ALIAS FOR $17;
    v_virus_name          ALIAS FOR $18;
    v_virus_type          ALIAS FOR $19;
    v_qa_id               ALIAS FOR $20;
    v_qa_live_ttl         ALIAS FOR $21;
    v_aa_id               ALIAS FOR $22;
    v_aa_live_ttl         ALIAS FOR $23;
    v_org_msg_id          ALIAS FOR $24;
    v_multi_virus         ALIAS FOR $25;
    v_ref_num             ALIAS FOR $26;
    v_filter_type_virus         bit;
    v_filter_type_spam          bit;
    v_filter_type_attachment    bit;
    v_filter_type_content       bit;
    v_filter_type_size          bit;
    v_filter_type_other         bit;
    v_filter_type_malformed     bit;
    v_filter_type_unscanned     bit;
    v_filter_type_phishing      bit;
    v_filter_type_spyware       bit;
    v_act_deleted               bit;
    v_act_quarantined           bit;
    v_act_delivered             bit;
    v_act_archived              bit;
    v_act_rcptchanged           bit;
    v_act_stamped               bit;
    v_act_attachments_deleted   bit;
    v_act_subjects_tagged       bit;
    v_act_delivery_postponed    bit;
    v_act_notifications_sent    bit;
    v_act_cleaned           bit;
    v_act_bcc           bit;
    v_act_handoff                   bit;
    v_policy_event_id               int4;
        v_len               int;
    v_pos               int;
    v_str               varchar(2048);
BEGIN  
    /*parse to get filter type*/
    v_filter_type_virus      := substring(v_filter_types from 1 for 1);
    v_filter_type_spam       := substring(v_filter_types from 2 for 1);
    v_filter_type_attachment := substring(v_filter_types from 3 for 1);
    v_filter_type_content    := substring(v_filter_types from 4 for 1);
    v_filter_type_size       := substring(v_filter_types from 5 for 1);
    v_filter_type_other      := substring(v_filter_types from 6 for 1);
    v_filter_type_malformed  := substring(v_filter_types from 7 for 1);
    v_filter_type_unscanned  := substring(v_filter_types from 8 for 1);
    v_filter_type_phishing   := substring(v_filter_types from 9 for 1);
    v_filter_type_spyware    := substring(v_filter_types from 10 for 1);
    /*parse to get action */
    v_act_deleted            := substring(v_msg_actions from 1 for 1);
    v_act_quarantined        := substring(v_msg_actions from 2 for 1);
    v_act_delivered          := substring(v_msg_actions from 3 for 1);
    v_act_archived           := substring(v_msg_actions from 4 for 1);
    v_act_handoff            := substring(v_msg_actions from 5 for 1); 
    v_act_rcptchanged        := substring(v_msg_actions from 6 for 1);
    v_act_stamped            := substring(v_msg_actions from 7 for 1);
    v_act_attachments_deleted:= substring(v_msg_actions from 8 for 1);
    v_act_subjects_tagged    := substring(v_msg_actions from 9 for 1);
    v_act_delivery_postponed := substring(v_msg_actions from 10 for 1);
    v_act_notifications_sent := substring(v_msg_actions from 11 for 1);
    v_act_cleaned            := substring(v_msg_actions from 12 for 1);
    v_act_bcc                := substring(v_msg_actions from 13 for 1);
    v_policy_event_id        := nextval(''''''''tb_policy_event_'' || nHourSeq || ''_policy_event_id_seq'''''''');
    /*1.add a row into tb_policy_event, msg_id&scanner_id may be duplicate*/
    insert into tb_policy_event_'' || nHourSeq || '' values (v_policy_event_id,
        v_msg_id, v_scanner_id, v_smtp_sender, v_smtp_rcpt, v_msg_subject, v_msg_route, v_msg_size, v_rule_name, v_time_date, 
        v_filter_type_virus, v_filter_type_spyware, v_filter_type_spam, v_filter_type_attachment, 
        v_filter_type_content, v_filter_type_size, v_filter_type_other, v_filter_type_malformed, 
        v_filter_type_unscanned, v_filter_type_phishing, v_filter_description, v_act_deleted, v_act_quarantined,
        v_act_delivered, v_act_archived, v_act_rcptchanged, v_act_stamped, v_act_attachments_deleted, 
        v_act_subjects_tagged, v_act_delivery_postponed, v_act_notifications_sent, v_act_cleaned, v_act_bcc, 
        v_act_handoff, v_spam_score, v_spam_category_id, v_spam_sensitive, v_attach_name, v_attach_type,    
        v_virus_name, v_virus_type, v_org_msg_id);
    if not found then
        /*could not insert tb_policy_event*/
        return -1;
    end if; 

        /*2.add a row into tb_quatantine if v_act_quarantined = 1*/
    if v_act_quarantined = ''''''''1'''''''' then
        insert into tb_quarantine_'' || nHourSeq || '' (policy_event_id, 
                        msg_id, scanner_id, smtp_sender, smtp_rcpt, msg_subject, msg_route, msg_size, qa_id, rule_name, time_date, 
            filter_type_virus, filter_type_spyware, filter_type_spam, filter_type_attachment,
            filter_type_content, filter_type_size, filter_type_other, filter_type_malformed,
            filter_type_unscanned, filter_type_phishing, attach_name, qt_live_ttl, ref_num, original_msg_id) 
            values (v_policy_event_id, v_msg_id, v_scanner_id, v_smtp_sender, v_smtp_rcpt, v_msg_subject, v_msg_route, v_msg_size,
            v_qa_id, v_rule_name, v_time_date, 
            v_filter_type_virus, v_filter_type_spyware, v_filter_type_spam, v_filter_type_attachment,
            v_filter_type_content, v_filter_type_size, v_filter_type_other, v_filter_type_malformed,
            v_filter_type_unscanned, v_filter_type_phishing, v_attach_name, v_qa_live_ttl, v_ref_num, v_org_msg_id);
        if not found then
            /*could not insert tb_quarantine*/
            return -2;
        end if;
    end if;
    
            /*3.add a row into tb_archive if v_act_archived = 1*/
    if v_act_archived = ''''''''1'''''''' then
        insert into tb_archive_'' || nHourSeq || '' (policy_event_id, 
        msg_id, scanner_id, smtp_sender, smtp_rcpt, msg_subject, msg_route, msg_size, qa_id, rule_name, time_date, 
        filter_type_virus, filter_type_spyware, filter_type_spam, filter_type_attachment, filter_type_content,
        filter_type_size, filter_type_other, filter_type_malformed, filter_type_unscanned,
        filter_type_phishing, attach_name, qt_live_ttl, original_msg_id) values (v_policy_event_id,
        v_msg_id, v_scanner_id, v_smtp_sender, v_smtp_rcpt, v_msg_subject, v_msg_route, v_msg_size, v_aa_id, v_rule_name, v_time_date, 
        v_filter_type_virus, v_filter_type_spyware, v_filter_type_spam, v_filter_type_attachment, v_filter_type_content,
        v_filter_type_size, v_filter_type_other, v_filter_type_malformed, v_filter_type_unscanned,
        v_filter_type_phishing, v_attach_name, v_aa_live_ttl, v_org_msg_id);
        if not found then
            /*could not insert tb_archive*/
            return -3;
        end if;
    end if;

    /*4.add rows into tb_splited_virus_event if v_multi_virus = 1*/
    if v_multi_virus = ''''''''1'''''''' then
        v_len := length(v_virus_name);
        v_str := substring(v_virus_name from 2 for v_len-1);
        v_pos := position('''''''';'''''''' in v_str);
        while v_pos > 0 loop
            insert into tb_splited_virus_event (time_date, virus_name) 
                values (v_time_date, substring(v_str from 1 for (v_pos - 1)));
            v_str := substring(v_str from v_pos + 1 for v_len - v_pos);
            v_len := length(v_str);
            v_pos := position('''''''';'''''''' in v_str);
        end loop;
        if trim(both from v_str) <> '''''''''''''''' then
            insert into tb_splited_virus_event (time_date, virus_name) values (v_time_date, v_str);
        end if;
    end if;
    
    /*5.add a row into tb_postpone if v_act_delivery_postponed = 1*/
  	if v_act_delivery_postponed = ''''''''1'''''''' then
    		insert into tb_postpone values (v_msg_id, v_scanner_id, v_qa_id, v_smtp_sender, v_smtp_rcpt, 
    		    v_msg_subject, v_msg_route, v_msg_size, v_rule_name, v_time_date, 
      			v_filter_type_virus, v_filter_type_spyware, v_filter_type_spam, v_filter_type_attachment,
      			v_filter_type_content, v_filter_type_size, v_filter_type_other, v_filter_type_malformed,
      			v_filter_type_unscanned, v_filter_type_phishing, v_attach_name, v_qa_live_ttl, v_ref_num);
    		if not found then
      			/*could not insert tb_postpone*/
      			return -6;
    		end if;	
  	end if;


    return 0;
END;
'''' LANGUAGE ''''plpgsql'''';'';
    EXECUTE szCreateTableCmd;

    nTmp := nTmp + 1;
    end loop;

    return 0;
END;
' LANGUAGE plpgsql;

select CreateLogImportFunc(0, 192);

CREATE TABLE tb_top10_triggered_rule
(
  starttime timestamptz NOT NULL,
  endtime timestamptz NOT NULL,
  rule_name varchar(256) NOT NULL,
  triggered_count int8 NOT NULL,
  CONSTRAINT pk_top10_triggered_rule PRIMARY KEY (starttime, rule_name)
)
WITHOUT OIDS;
  
CREATE TABLE tb_top10_virus_code
(
  starttime timestamptz NOT NULL,
  endtime timestamptz NOT NULL,
  virus_name varchar(256) NOT NULL,
  detected_count int8 NOT NULL,
  CONSTRAINT pk_top10_virus_code PRIMARY KEY (starttime, virus_name)
)
WITHOUT OIDS;

CREATE OR REPLACE FUNCTION CheckTableId (int, timestamptz) RETURNS integer AS '
DECLARE
    v_hour_num       ALIAS FOR $1;
    v_starttime      ALIAS FOR $2;
    v_table_id       int;
BEGIN

    select into v_table_id table_id from tb_log_table_info where hour_num = v_hour_num;
    if not found then
        select into v_table_id table_id from tb_log_table_info where hour_num = -1 order by table_id asc limit 1;
        if found then
            update tb_log_table_info set hour_num = v_hour_num, time_start = v_starttime,
                time_end = v_starttime + interval''1 hour'', trafficsum_bookmark = 0, top100data_bookmark = 0,
                reportsum_bookmark = 0, indexed = ''0'', summarized = ''0'', trafficsum_gened = ''0'', top100data_gened = ''0''
                where table_id = v_table_id;
            if found then
                return v_table_id;
            end if;
        end if;
    else
        return v_table_id;
    end if;
    
    return -1; /* cannot create new tables */
END;
' LANGUAGE plpgsql;
