CREATE OR REPLACE FUNCTION int2interval(int4)
  RETURNS "interval" AS
'declare

BEGIN 
Return $1;
END;
'
  LANGUAGE 'plpgsql' VOLATILE;

-- DROP FUNCTION ipstring2int("varchar");

CREATE OR REPLACE FUNCTION ipstring2int("varchar")
  RETURNS int8 AS
'declare IPString alias for $1;
IPv4 int8;
part int4;
Begin
	IPv4 := 0;
	part := split_part(IPString ,''.'',1) as int4;
	IPv4 := IPv4*256 + part;
	part := split_part(IPString ,''.'',2) as int4;
	IPv4 := IPv4*256 + part;	
	part := split_part(IPString ,''.'',3) as int4;
	IPv4 := IPv4*256 + part;
	part := split_part(IPString ,''.'',4) as int4;
	IPv4 := IPv4*256 + part;
	RETURN IPv4;
End;'
  LANGUAGE 'plpgsql' VOLATILE;

CREATE OR REPLACE FUNCTION ipint2string(int8)
  RETURNS "varchar" AS
'declare
	IPv4 alias for $1;
	binIPv4 int8;
	firstByte int8;
	ret varchar(20);
BEGIN 
ret:='''';
binIPv4:=IPv4;
firstByte:=binIPv4>>24;
if firstByte < 0 then
firstByte:=firstByte+256;
end if;
ret:=ret||firstByte||''.'';
binIPv4:=binIPv4&16777215;
ret:=ret||(binIPv4>>16)||''.'';
binIPv4:=binIPv4&65535;
ret:=ret||(binIPv4>>8)||''.'';
binIPv4:=binIPv4&255;
ret:=ret||(binIPv4>>0);
return ret;
END;
'
  LANGUAGE 'plpgsql' VOLATILE;
  
CREATE OR REPLACE FUNCTION msg2cachedip()
  RETURNS "trigger" AS
'
 DECLARE
 dha_setting RECORD;
 bounced_setting RECORD;
 _isspam int2;
 _isvirus int2;	
 _isdha int2;
 _isbounced int2;
BEGIN
	_isdha:=0;
	_isbounced:=0;
	_isspam:=0;
	_isvirus=0;
	SELECT INTO dha_setting * FROM t_type_setting WHERE type=1 ;
	IF NEW.Total_rcptto_count > dha_setting.recipients
		THEN 
		IF (NEW.Unknown_rcptto_count>0)
		THEN 
		_isdha:=1;
		ELSE
			IF (NEW.Noexist_rcptto_count>dha_setting.nonexistrcpt)
			THEN 
			_isdha:=1;
			END iF;
		END iF;
	END IF;
	IF NEW.Noexist_rcptto_count > 0
		THEN 
		_isbounced:=1;
	END IF	;

	if (NEW.msgisspam=true)
				Then 
	_isspam:=1;
	end if;
	
	if (NEW.msgcontainvirus=true) 
				Then 
	_isvirus:=1;
	end if;
	
	INSERT into  t_cachedip (ip,isspam,isvirus,isdha,isbounced,time) values (NEW.upstreammta_ip,_isspam,_isvirus,_isdha,_isbounced,NEW.sessionstarttime) ;
	--return NEW;
	return NULL;
 END;
'
  LANGUAGE 'plpgsql' VOLATILE;  

CREATE OR REPLACE FUNCTION checkruleall(int4, int8, "timestamptz")
  RETURNS int4 AS
'
declare
setting RECORD;
current RECORD;
iprule RECORD;
iplog RECORD;
tt timestamptz;
newip RECORD;
BEGIN 

select into newip * from t_cachedip_tmp2 where ip=$2 limit 1;
IF  FOUND THEN

SELECT INTO setting * FROM t_type_setting WHERE type=1;

IF  FOUND THEN
tt=$3-int2interval(setting.duration)*3600;

SELECT INTO current sum(totalmsg) as allcount,sum(isdha) as count1,sum(isvirus) as count2,
sum(isspam) as count3,sum(isbounced) as count4  from t_cachedip_tmp 
where  ip=$2 and time> tt and time<=$3;
end if;
if (current.allcount is null) then  current.allcount=0 ;end if;

SELECT INTO setting * FROM t_type_setting WHERE type=1 and enable=true;
IF  FOUND THEN

if (current.allcount<>0) then
if ((current.count1*100.0/current.allcount) > setting.rate ) and (current.allcount >setting.totalmail)
then 
select into iprule * from t_iprule where ip=$2;
if NOT FOUND then
select into iprule * from t_iprule where ipint2string(ip)=ipint2string($2);
if NOT FOUND then
select into iplog ''matched: ''||current.count1||'' all: ''||current.allcount||'' time ''||$3||'' - ''||($3-int2interval(setting.duration)*3600) as log;
insert into t_iprule ( enable,ip,type,action,createtime,comments) values (true,$2,1,setting.action,LOCALTIMESTAMP(0),iplog.log);
delete  from t_cachedip_tmp2 where ip=$2;
	Return 1;
end if;
end if;
end if;
end if;
end if;

SELECT INTO setting * FROM t_type_setting WHERE type=2 and enable=true;
IF  FOUND THEN

if (current.allcount<>0) then
if ((current.count2*100.0/current.allcount) > setting.rate ) and (current.allcount >setting.totalmail)
then 
select into iprule * from t_iprule where ip=$2;
if NOT FOUND then
select into iprule * from t_iprule where ipint2string(ip)=ipint2string($2);
if NOT FOUND then
select into iplog ''matched: ''||current.count2||'' all: ''||current.allcount||'' time ''||$3||'' - ''||($3-int2interval(setting.duration)*3600) as log;
insert into t_iprule ( enable,ip,type,action,createtime,comments) values (true,$2,2,setting.action,LOCALTIMESTAMP(0),iplog.log);
delete  from t_cachedip_tmp2 where ip=$2;
	Return 1;
end if;
end if;
end if;
end if;
end if;


SELECT INTO setting * FROM t_type_setting WHERE type=3 and enable=true;
IF  FOUND THEN

if (current.allcount<>0) then
if ((current.count3*100.0/current.allcount) > setting.rate ) and (current.allcount >setting.totalmail)
then 
select into iprule * from t_iprule where ip=$2;
if NOT FOUND then
select into iprule * from t_iprule where ipint2string(ip)=ipint2string($2);
if NOT FOUND then
select into iplog ''matched: ''||current.count3||'' all: ''||current.allcount||'' time ''||$3||'' - ''||($3-int2interval(setting.duration)*3600) as log;
insert into t_iprule ( enable,ip,type,action,createtime,comments) values (true,$2,3,setting.action,LOCALTIMESTAMP(0),iplog.log);
delete  from t_cachedip_tmp2 where ip=$2;
	Return 1;
end if;
end if;
end if;
end if;
end if;

SELECT INTO setting * FROM t_type_setting WHERE type=4 and enable=true;
IF  FOUND THEN

if (current.allcount<>0) then
if ((current.count4*100.0/current.allcount) > setting.rate ) and (current.allcount >setting.totalmail)
then 
select into iprule * from t_iprule where ip=$2;
if NOT FOUND then
select into iprule * from t_iprule where ipint2string(ip)=ipint2string($2);
if NOT FOUND then
select into iplog ''matched: ''||current.count4||'' all: ''||current.allcount||'' time ''||$3||'' - ''||($3-int2interval(setting.duration)*3600) as log;
insert into t_iprule ( enable,ip,type,action,createtime,comments) values (true,$2,4,setting.action,LOCALTIMESTAMP(0),iplog.log);
delete  from t_cachedip_tmp2 where ip=$2;
	Return 1;
end if;
end if;
end if;
end if;
end if;




End if; --check if in t_cachedip_tmp2
Return 0;	
END;
'
  LANGUAGE 'plpgsql' VOLATILE;
  
CREATE OR REPLACE FUNCTION checkrulebytype(int4, int8, "timestamptz")
  RETURNS int4 AS
'
declare
setting RECORD;
current RECORD;
iprule RECORD;
iplog RECORD;
tt timestamptz;
newip RECORD;
BEGIN 

select into newip * from t_cachedip_tmp2 where ip=$2 limit 1;
IF  FOUND THEN

SELECT INTO setting * FROM t_type_setting WHERE type=$1 and enable=true;

IF  FOUND THEN
tt=$3-int2interval(setting.duration)*3600;
if $1=1 then
SELECT INTO current sum(totalmsg) as allcount,sum(isdha) as count from t_cachedip_tmp 
where  ip=$2 and time> tt and time<=$3;
end if;

if $1=2 then
SELECT INTO current sum(totalmsg) as allcount,sum(isvirus) as count from t_cachedip_tmp 
where  ip=$2 and time> tt and time<=$3;
end if;

if $1=3 then
SELECT INTO current sum(totalmsg) as allcount,sum(isspam) as count from t_cachedip_tmp 
where  ip=$2 and time> tt and time<=$3;
end if;

if $1=4 then
SELECT INTO current sum(totalmsg) as allcount,sum(isbounced) as count from t_cachedip_tmp 
where  ip=$2 and time> tt and time<=$3;
end if;

if (current.allcount is null) then  current.allcount=0 ;end if;
if (current.allcount<>0) then
if ((current.count*100.0/current.allcount) > setting.rate ) and (current.allcount >setting.totalmail)
then 
select into iprule * from t_iprule where ip=$2;
if NOT FOUND then
select into iprule * from t_iprule where ipint2string(ip)=ipint2string($2);
if NOT FOUND then
select into iplog ''matched: ''||current.count||'' all: ''||current.allcount||'' time ''||$3||'' - ''||($3-int2interval(setting.duration)*3600) as log;
insert into t_iprule ( enable,ip,type,action,createtime,comments) values (true,$2,$1,setting.action,LOCALTIMESTAMP(0),iplog.log);
delete  from t_cachedip_tmp2 where ip=$2;
	Return 1;
end if;
end if;
end if;
end if;

end if;

End if; --check if in t_cachedip_tmp2
Return 0;	
END;
'
  LANGUAGE 'plpgsql' VOLATILE;
  
CREATE OR REPLACE FUNCTION checksumruleall(int4, int8, "timestamptz", int4, int4, int4, int4, int4)
  RETURNS int4 AS
'
declare
setting RECORD;
current RECORD;
iprule RECORD;
iplog RECORD;
tt timestamptz;
sum1 ALIAS FOR $4;
sum2 ALIAS FOR $5;
sum3 ALIAS FOR $6;
sum4 ALIAS FOR $7;
total ALIAS FOR $8;
BEGIN 

SELECT INTO setting * FROM t_type_setting WHERE type=1;

IF  FOUND THEN
tt=$3-int2interval(setting.duration)*3600;

SELECT INTO current sum(totalmsg) as allcount,sum(isdha) as count1,sum(isvirus) as count2,
sum(isspam) as count3,sum(isbounced) as count4  from t_cachedip_tmp 
where  ip=$2 and time> tt and time<=$3;
end if;

if (current.allcount is null) then  current.allcount=0 ;end if;
if (current.count1 is null) then current.count1=0 ;end if;
if (current.count2 is null) then current.count2=0 ;end if;
if (current.count3 is null) then current.count3=0 ;end if;
if (current.count4 is null) then current.count4=0 ;end if;

SELECT INTO setting * FROM t_type_setting WHERE type=1 and enable=true;
IF  FOUND THEN
current.allcount=current.allcount+total;
current.count1=current.count1+sum1;

if (current.allcount<>0) then
if ((current.count1*100.0/current.allcount) > setting.rate ) and (current.allcount >setting.totalmail)
then 
select into iprule * from t_iprule where ip=$2;
if NOT FOUND then
select into iprule * from t_iprule where ipint2string(ip)=ipint2string($2);
if NOT FOUND then
	Return 1;
end if;
end if;
end if;
end if;
end if;

SELECT INTO setting * FROM t_type_setting WHERE type=2 and enable=true;
IF  FOUND THEN
current.count2=current.count2+sum2;
if (current.allcount<>0) then
if ((current.count2*100.0/current.allcount) > setting.rate ) and (current.allcount >setting.totalmail)
then 
select into iprule * from t_iprule where ip=$2;
if NOT FOUND then
select into iprule * from t_iprule where ipint2string(ip)=ipint2string($2);
if NOT FOUND then
	Return 1;
end if;
end if;
end if;
end if;
end if;


SELECT INTO setting * FROM t_type_setting WHERE type=3 and enable=true;
IF  FOUND THEN
current.count3=current.count3+sum3;
--RAISE NOTICE  ''all % count % % %'',current.allcount,current.count3,setting.rate,setting.totalmail;
if (current.allcount<>0) then
if ((current.count3*100.0/current.allcount) > setting.rate ) and (current.allcount >setting.totalmail)
then 
select into iprule * from t_iprule where ip=$2;
if NOT FOUND then
select into iprule * from t_iprule where ipint2string(ip)=ipint2string($2);
if NOT FOUND then
Return 1;
end if;
end if;
end if;
end if;
end if;

SELECT INTO setting * FROM t_type_setting WHERE type=4 and enable=true;
IF  FOUND THEN
current.count4=current.count4+sum4;
if (current.allcount<>0) then
if ((current.count4*100.0/current.allcount) > setting.rate ) and (current.allcount >setting.totalmail)
then 
select into iprule * from t_iprule where ip=$2;
if NOT FOUND then
select into iprule * from t_iprule where ipint2string(ip)=ipint2string($2);
if NOT FOUND then
	Return 1;
end if;
end if;
end if;
end if;
end if;





Return 0;	
END;
'
  LANGUAGE 'plpgsql' VOLATILE;
  
CREATE OR REPLACE FUNCTION checksumrulebytype(int4, int8, "timestamptz", int4, int4)
  RETURNS int4 AS
'
declare
setting RECORD;
current RECORD;
iprule RECORD;
iplog RECORD;
tt timestamptz;
sumnumber ALIAS FOR $4;
total ALIAS FOR $5;
BEGIN 

SELECT INTO setting * FROM t_type_setting WHERE type=$1 and enable=true;

IF  FOUND THEN
tt=$3-int2interval(setting.duration)*3600;
if $1=1 then
SELECT INTO current sum(totalmsg) as allcount,sum(isdha) as count from t_cachedip_tmp 
where  ip=$2 and time> tt and time<=$3;
end if;

if $1=2 then
SELECT INTO current sum(totalmsg) as allcount,sum(isvirus) as count from t_cachedip_tmp 
where  ip=$2 and time> tt and time<=$3;
end if;

if $1=3 then
SELECT INTO current sum(totalmsg) as allcount,sum(isspam) as count from t_cachedip_tmp 
where  ip=$2 and time> tt and time<=$3;
end if;

if $1=4 then
SELECT INTO current sum(totalmsg) as allcount,sum(isbounced) as count from t_cachedip_tmp 
where  ip=$2 and time> tt and time<=$3;
end if;
if (current.allcount is null) then  current.allcount=0 ;end if;
if (current.count is null) then current.count=0 ;end if;
current.allcount=current.allcount+total;
current.count=current.count+sumnumber;
if (current.allcount<>0) then
if ((current.count*100.0/current.allcount) > setting.rate ) and (current.allcount >setting.totalmail)
then 
select into iprule * from t_iprule where ip=$2;
if NOT FOUND then
select into iprule * from t_iprule where ipint2string(ip)=ipint2string($2);
if NOT FOUND then
	Return 1;
end if;
end if;
end if;
end if;

end if;
Return 0;	
END;
'
  LANGUAGE 'plpgsql' VOLATILE;
  
CREATE OR REPLACE FUNCTION cleanup_cachediptmp()
  RETURNS int4 AS
'
declare
setting RECORD;
cacheiptmp RECORD;
tt timestamptz;
BEGIN 
	SELECT INTO setting max(duration)as max_duration from t_type_setting;
	SELECT INTO cacheiptmp max(time)as max_time,min(time) as min_time from t_cachedip_tmp;
	tt=cacheiptmp.max_time-int2interval(setting.max_duration)*3600;
	if(cacheiptmp.min_time+int2interval(600))<tt then
	delete from t_cachedip_tmp where time< tt;
	end if;
return 0;
END;
'
  LANGUAGE 'plpgsql' VOLATILE;
  
CREATE OR REPLACE FUNCTION rule_check()
  RETURNS "trigger" AS
'declare

setting record; 
setting1 record; 
setting2 record; 
setting3 record; 
setting4 record; 
BEGIN 

SELECT into setting duration from t_type_setting WHERE  enable=true limit 1;
if FOUND then

SELECT into setting1 duration  from t_type_setting WHERE type=1 and enable=true;
SELECT into setting2 duration  from t_type_setting WHERE type=2 and enable=true;
SELECT into setting3 duration  from t_type_setting WHERE type=3 and enable=true;
SELECT into setting4 duration  from t_type_setting WHERE type=4 and enable=true;



if (setting1.duration=setting2.duration)
 and (setting2.duration=setting3.duration)
 and (setting3.duration=setting4.duration)
then
if checkruleall(0,NEW.ip,NEW.time)=1 then return NEW;end if;	
else
if checkrulebytype(1,NEW.ip,NEW.time)=1 then return NEW;end if;	
if checkrulebytype(2,NEW.ip,NEW.time)=1 then return NEW;end if;	
if checkrulebytype(3,NEW.ip,NEW.time)=1 then return NEW;end if;	
if checkrulebytype(4,NEW.ip,NEW.time)=1 then return NEW;end if;	

end if;

end if;
Return NEW;
END;
'
  LANGUAGE 'plpgsql' VOLATILE;
  
CREATE OR REPLACE FUNCTION rulesum_check()
  RETURNS "trigger" AS
'declare

setting record; 
setting1 record; 
setting2 record; 
setting3 record; 
setting4 record; 
BEGIN 

SELECT into setting duration from t_type_setting WHERE  enable=true limit 1;
if FOUND then

SELECT into setting1 duration  from t_type_setting WHERE type=1 and enable=true;
SELECT into setting2 duration  from t_type_setting WHERE type=2 and enable=true;
SELECT into setting3 duration  from t_type_setting WHERE type=3 and enable=true;
SELECT into setting4 duration  from t_type_setting WHERE type=4 and enable=true;



if (setting1.duration=setting2.duration)
 and (setting2.duration=setting3.duration)
 and (setting3.duration=setting4.duration)
then
if checksumruleall(0,NEW.ip,NEW.time,NEW.isdha,NEW.isvirus,NEW.isspam,NEW.isbounced,NEW.total)=1 then return NEW;end if;	
else
if checksumrulebytype(1,NEW.ip,NEW.time,NEW.isdha,NEW.total)=1 then return NEW;end if;	
if checksumrulebytype(2,NEW.ip,NEW.time,NEW.isvirus,NEW.total)=1 then return NEW;end if;	
if checksumrulebytype(3,NEW.ip,NEW.time,NEW.isspam,NEW.total)=1 then return NEW;end if;	
if checksumrulebytype(4,NEW.ip,NEW.time,NEW.isbounced,NEW.total)=1 then return NEW;end if;	

end if;

end if;
Return NULL;
END;
'
  LANGUAGE 'plpgsql' VOLATILE;
  
CREATE TABLE t_cachedip_tmp2
(
  ip int8,
  isspam int4,
  isdha int4,
  isvirus int4,
  isbounced int4,
  total int4,
  "time" timestamp with time zone
);
          
CREATE TRIGGER autorulesum
  BEFORE INSERT
  ON t_cachedip_tmp2
  FOR EACH ROW
  EXECUTE PROCEDURE rulesum_check();


CREATE TRIGGER autorule
  AFTER INSERT
  ON t_cachedip_tmp
  FOR EACH ROW
  EXECUTE PROCEDURE rule_check();


CREATE TRIGGER cachedip
  BEFORE INSERT
  ON t_messages
  FOR EACH ROW
  EXECUTE PROCEDURE msg2cachedip();


CREATE INDEX idx_flag
  ON t_cachedip
  USING btree
  (processflag);

CREATE INDEX idx_ip
  ON t_cachedip
  USING btree
  (ip);

CREATE INDEX idx_time
  ON t_cachedip
  USING btree
  ("time");

CREATE INDEX idx_iptime
  ON t_cachedip_tmp
  USING btree
  (ip, "time");
