CREATE OR REPLACE FUNCTION GenHourlyPolicySum(int, timestamptz, timestamptz) RETURNS integer AS '
DECLARE
    msg_cnt_total    INTEGER;
    msg_cnt_in       INTEGER;
    msg_cnt_out      INTEGER;
    msg_size_total   float;
    msg_size_in      float;
    msg_size_out     float;
    virus_total      INTEGER;
    virus_incoming   INTEGER;
    virus_outgoing   INTEGER;
    spy_total        INTEGER;
    spy_incoming     INTEGER;
    spy_outgoing     INTEGER;
    spam_total       INTEGER;
    spam_incoming    INTEGER;
    spam_outgoing    INTEGER;
    phish_total      INTEGER;
    phish_incoming   INTEGER;
    phish_outgoing   INTEGER;
    attach_total     INTEGER;
    attach_incoming  INTEGER;
    attach_outgoing  INTEGER;
    size_total       INTEGER;
    size_incoming    INTEGER;
    size_outgoing    INTEGER;
    content_total    INTEGER;
    content_incoming INTEGER;
    content_outgoing INTEGER;
    other_total      INTEGER;
    other_incoming   INTEGER;
    other_outgoing   INTEGER;
    except_total     INTEGER;
    except_incoming  INTEGER;
    except_outgoing  INTEGER;
    cnt_ipprof_conn  INTEGER;
    cnt_ipprof_block INTEGER;
    cnt_virus        INTEGER;
    cnt_spam         INTEGER;
    cnt_bounce       INTEGER;
    cnt_dha          INTEGER;
    cnt_manual       INTEGER;
    cnt_nrs_conn     INTEGER;
    cnt_nrs_block    INTEGER;
    spamact_quar     INTEGER;
    spamact_del      INTEGER;
    spamact_subtag   INTEGER;
    spamact_other    INTEGER;
    spamact_total_size  float;
    spamact_quar_size   float;
    spamact_del_size    float;
    spamact_subtag_size float;
    spamact_other_size  float;
    virusact_del     INTEGER;
    virusact_quar    INTEGER;
    virusact_clean   INTEGER;
    virusact_attdel  INTEGER;    
    rec_count        RECORD;
    table_id         ALIAS FOR $1;
    v_starttime      ALIAS FOR $2;
    v_endtime        ALIAS FOR $3;
    v_tmp            timestamptz;
    sqlcmd           text;
BEGIN
    msg_cnt_total    := 0;
    msg_cnt_in       := 0;
    msg_cnt_out      := 0;
    msg_size_total   := 0;
    msg_size_in      := 0;
    msg_size_out     := 0;
    virus_total      := 0;
    virus_incoming   := 0;
    virus_outgoing   := 0;
    spy_total        := 0;
    spy_incoming     := 0;
    spy_outgoing     := 0;
    spam_total       := 0;
    spam_incoming    := 0;
    spam_outgoing    := 0;
    phish_total      := 0;
    phish_incoming   := 0;
    phish_outgoing   := 0;
    attach_total     := 0;
    attach_incoming  := 0;
    attach_outgoing  := 0;
    size_total       := 0;
    size_incoming    := 0;
    size_outgoing    := 0;
    content_total    := 0;
    content_incoming := 0;
    content_outgoing := 0;
    other_total      := 0;
    other_incoming   := 0;
    other_outgoing   := 0;
    except_total     := 0;
    except_incoming  := 0;
    except_outgoing  := 0;
    spamact_quar     := 0;
    spamact_del      := 0;
    spamact_subtag   := 0;
    spamact_other    := 0;
    spamact_total_size  := 0;
    spamact_quar_size   := 0;
    spamact_del_size    := 0;
    spamact_subtag_size := 0;
    spamact_other_size  := 0;
    virusact_del     := 0;
    virusact_quar    := 0;
    virusact_clean   := 0;
    virusact_attdel  := 0; 
    
    sqlcmd := ''select msg_route, count(*) as cnt, sum(msg_size) as sum,
            sum(filter_type_virus) as virus, sum(filter_type_spyware) as spyware,
            sum(filter_type_spam) as spam, sum(filter_type_phishing) as phishing,
            sum(filter_type_attachment) as attachment, sum(filter_type_size) as size,
            sum(filter_type_content) as content, sum(filter_type_other) as other,
            sum(filter_type_except) as exception,
            sum(spamact_quar) as sa_quar, sum(spamact_del) as sa_del,
            sum(spamact_subtag) as sa_subtag, sum(spamact_other) as sa_other,
            sum(filter_type_spam*msg_size) as sa_total_size,
            sum(spamact_quar*msg_size) as sa_quar_size, sum(spamact_del*msg_size) as sa_del_size,
            sum(spamact_subtag*msg_size) as sa_subtag_size, sum(spamact_other*msg_size) as sa_other_size,
            sum(virusact_del) as va_del, sum(virusact_quar) as va_quar,
            sum(virusact_clean) as va_clean, sum(virusact_attdel) as va_attdel from
            (
            select
            case when sum(pe.filter_type_virus::integer) >0 then 1 else 0 end       as filter_type_virus,
            case when sum(pe.filter_type_spyware::integer) >0 then 1 else 0 end     as filter_type_spyware,
            case when sum(pe.filter_type_spam::integer) >0 then 1 else 0 end        as filter_type_spam,
            case when sum(pe.filter_type_phishing::integer) >0 then 1 else 0 end    as filter_type_phishing,
            case when sum(pe.filter_type_attachment::integer) >0 then 1 else 0 end  as filter_type_attachment,
            case when sum(pe.filter_type_size::integer) >0 then 1 else 0 end        as filter_type_size,
            case when sum(pe.filter_type_content::integer) >0 then 1 else 0 end     as filter_type_content,
            case when sum(pe.filter_type_other::integer) >0 then 1 else 0 end       as filter_type_other,
            case when sum((pe.filter_type_malformed|pe.filter_type_unscanned)::integer) >0 then 1 else 0 end   as filter_type_except,
            case when sum((pe.filter_type_spam&pe.act_quarantined)::integer) >0 then 1 else 0 end              as spamact_quar,
            case when sum((pe.filter_type_spam&pe.act_deleted)::integer) >0 then 1 else 0 end                  as spamact_del,
            case when sum((pe.filter_type_spam&pe.act_subjects_tagged)::integer) >0 then 1 else 0 end          as spamact_subtag,
            case when sum((pe.filter_type_spam&(((~pe.act_quarantined)&(~pe.act_deleted)&(~pe.act_subjects_tagged))|pe.act_delivered|pe.act_archived|pe.act_rcptchanged|pe.act_stamped|pe.act_attachments_deleted|pe.act_delivery_postponed|pe.act_notifications_sent|pe.act_cleaned|pe.act_bcc|pe.act_handoff))::integer) >0 then 1 else 0 end          as spamact_other,
            case when sum((pe.filter_type_virus&pe.act_deleted)::integer) >0 then 1 else 0 end                 as virusact_del,            
            case when sum((pe.filter_type_virus&pe.act_quarantined)::integer) >0 then 1 else 0 end             as virusact_quar,
            case when sum((pe.filter_type_virus&pe.act_cleaned)::integer) >0 then 1 else 0 end                 as virusact_clean,
            case when sum((pe.filter_type_virus&pe.act_attachments_deleted)::integer) >0 then 1 else 0 end     as virusact_attdel,
            msg_id                                                                  as msg_id,
            scanner_id                                                              as scanner_id,
            msg_route                                                               as msg_route,
            max(msg_size)                                                           as msg_size
            from tb_policy_event_'' || table_id || '' pe
            group by msg_id, scanner_id, msg_route
            ) as foo group by msg_route'';
    for rec_count in execute sqlcmd
    loop
        if rec_count.msg_route = 1 then
            msg_cnt_in := rec_count.cnt;
            msg_size_in := rec_count.sum;
            virus_incoming := rec_count.virus;
            spy_incoming := rec_count.spyware;
            spam_incoming := rec_count.spam;
            phish_incoming := rec_count.phishing;
            attach_incoming := rec_count.attachment;
            size_incoming := rec_count.size;
            content_incoming := rec_count.content;
            other_incoming := rec_count.other;
            except_incoming := rec_count.exception;
        else
            if rec_count.msg_route = 2 then
                msg_cnt_out := rec_count.cnt;
                msg_size_out := rec_count.sum;
                virus_outgoing := rec_count.virus;
                spy_outgoing := rec_count.spyware;
                spam_outgoing := rec_count.spam;
                phish_outgoing := rec_count.phishing;
                attach_outgoing := rec_count.attachment;
                size_outgoing := rec_count.size;
                content_outgoing := rec_count.content;
                other_outgoing := rec_count.other;
                except_outgoing := rec_count.exception;
            end if;
        end if;
        msg_cnt_total := msg_cnt_total + rec_count.cnt;
        msg_size_total := msg_size_total + rec_count.sum;     
        virus_total := virus_total + rec_count.virus;
        spy_total := spy_total + rec_count.spyware;
        spam_total := spam_total + rec_count.spam;
        phish_total := phish_total + rec_count.phishing;
        attach_total := attach_total + rec_count.attachment;
        size_total := size_total + rec_count.size;
        content_total := content_total + rec_count.content;
        other_total := other_total + rec_count.other;
        except_total := except_total + rec_count.exception;
        spamact_quar := spamact_quar + rec_count.sa_quar;
        spamact_del := spamact_del + rec_count.sa_del;
        spamact_subtag := spamact_subtag + rec_count.sa_subtag;
        spamact_other := spamact_other + rec_count.sa_other;
        spamact_total_size := spamact_total_size + rec_count.sa_total_size;
        spamact_quar_size := spamact_quar_size + rec_count.sa_quar_size;
        spamact_del_size := spamact_del_size + rec_count.sa_del_size;
        spamact_subtag_size := spamact_subtag_size + rec_count.sa_subtag_size;
        spamact_other_size := spamact_other_size + rec_count.sa_other_size;
        virusact_del := virusact_del + rec_count.va_del;
        virusact_quar := virusact_quar + rec_count.va_quar;
        virusact_clean := virusact_clean + rec_count.va_clean;
        virusact_attdel := virusact_attdel + rec_count.va_attdel;
    end loop;
    
    cnt_ipprof_conn  := 0;
    cnt_ipprof_block := 0;
    cnt_virus        := 0;
    cnt_spam         := 0;
    cnt_bounce       := 0;
    cnt_dha          := 0;
    cnt_manual       := 0;
    
    select into cnt_ipprof_conn sum(totalmsg) from t_cachedip where time >= v_starttime and time < v_endtime;
    if (cnt_ipprof_conn is null) then
        cnt_ipprof_conn := 0;
    end if;
    
    for rec_count in
        select type, sum(count) as cnt from t_blockedip where createtime >= v_starttime and createtime < v_endtime group by type
    loop
        if rec_count.type = 1 then
            cnt_dha := rec_count.cnt;
        end if;
        if rec_count.type = 2 then
            cnt_virus := rec_count.cnt;
        end if;
        if rec_count.type = 3 then
            cnt_spam := rec_count.cnt;
        end if;
        if rec_count.type = 4 then
            cnt_bounce := rec_count.cnt;
        end if;
        if rec_count.type = 5 then
            cnt_manual := rec_count.cnt;
        end if;
        cnt_ipprof_block := cnt_ipprof_block + rec_count.cnt;
    end loop;

    cnt_nrs_conn  := 0;
    cnt_nrs_block := 0;
    for rec_count in
        select action, sum(act_num) as cnt from tb_ip_event where time_date >= v_starttime and time_date < v_endtime group by action
    loop
        if rec_count.action = 0 then
            cnt_nrs_conn := rec_count.cnt;
        else
            if rec_count.action = 1 then
                cnt_nrs_block := rec_count.cnt;
            end if;
        end if;
    end loop;
    cnt_ipprof_conn := cnt_ipprof_conn + cnt_nrs_block;
    
    select into v_tmp starttime from tb_traffic_summary where starttime = v_starttime;
    if found then
       update tb_traffic_summary set starttime=v_starttime, endtime=v_endtime,
       msg_total=msg_cnt_total, msg_in=msg_cnt_in, msg_out=msg_cnt_out,
       msg_total_size=msg_size_total, msg_in_size=msg_size_in, msg_out_size=msg_size_out,
       pol_virus_total=virus_total, pol_virus_in=virus_incoming, pol_virus_out=virus_outgoing,
       pol_spyware_total=spy_total, pol_spyware_in=spy_incoming, pol_spyware_out=spy_outgoing,
       pol_spam_total=spam_total, pol_spam_in=spam_incoming, pol_spam_out=spam_outgoing,
       pol_phish_total=phish_total, pol_phish_in=phish_incoming, pol_phish_out=phish_outgoing,
       pol_attach_total=attach_total, pol_attach_in=attach_incoming, pol_attach_out=attach_outgoing,
       pol_size_total=size_total, pol_size_in=size_incoming, pol_size_out=size_outgoing,
       pol_content_total=content_total, pol_content_in=content_incoming, pol_content_out=content_outgoing,
       pol_other_total=other_total, pol_other_in=other_incoming, pol_other_out=other_outgoing,
       pol_exception_total=except_total, pol_exception_in=except_incoming, pol_exception_out=except_outgoing,
       pol_ipfilt_total=cnt_ipprof_block, pol_ipfilt_in=cnt_ipprof_block, pol_ipfilt_out=0,
       pol_nrs_total=cnt_nrs_block, pol_nrs_in=cnt_nrs_block, pol_nrs_out=0,
       ipfilt_conn=cnt_ipprof_conn, ipfilt_block=cnt_ipprof_block, ipfilt_virus=cnt_virus, ipfilt_spam=cnt_spam,
       ipfilt_bounce=cnt_bounce, ipfilt_dha=cnt_dha, ipfilt_manual=cnt_manual,
       nrs_conn=cnt_nrs_conn, nrs_block=cnt_nrs_block,
       spam_quar=spamact_quar, spam_del=spamact_del, spam_subtag=spamact_subtag, spam_other=spamact_other, spam_total_size=spamact_total_size,
       spam_quar_size=spamact_quar_size, spam_del_size=spamact_del_size, spam_subtag_size=spamact_subtag_size, spam_other_size=spamact_other_size,
       virus_del=virusact_del, virus_quar=virusact_quar, virus_clean=virusact_clean, virus_attdel=virusact_attdel
       where starttime = v_starttime;
       if not found then
           return -1;
       end if;
    else
       insert into tb_traffic_summary values (v_starttime, v_endtime,
       msg_cnt_total, msg_cnt_in, msg_cnt_out, msg_size_total, msg_size_in, msg_size_out,
       virus_total, virus_incoming, virus_outgoing, spy_total, spy_incoming, spy_outgoing,
       spam_total, spam_incoming, spam_outgoing, phish_total, phish_incoming, phish_outgoing,
       attach_total, attach_incoming, attach_outgoing, size_total, size_incoming, size_outgoing,
       content_total, content_incoming, content_outgoing, other_total, other_incoming, other_outgoing,
       except_total, except_incoming, except_outgoing,
       cnt_ipprof_block, cnt_ipprof_block, 0,
       cnt_nrs_block, cnt_nrs_block, 0,
       cnt_ipprof_conn, cnt_ipprof_block, cnt_virus, cnt_spam, cnt_bounce, cnt_dha, cnt_manual,
       cnt_nrs_conn, cnt_nrs_block,
       spamact_quar, spamact_del, spamact_subtag, spamact_other, spamact_total_size,
       spamact_quar_size, spamact_del_size, spamact_subtag_size, spamact_other_size,
       virusact_del, virusact_quar, virusact_clean, virusact_attdel);
       if not found then
           return -2;
       end if;
    end if;
    
    return 0;
END;
' LANGUAGE plpgsql;
