#!/bin/sh
PATH=/sbin:/usr/sbin:/bin:/usr/bin:$PATH
export PATH

PS_CMD="ps -ef --width 1000"

NRS_HOME=/opt/trend/nrs
export NRS_HOME
IMSS_HOME=$NRS_HOME
export IMSS_HOME

if [ -f /etc/trend-release ]; then
	LD_LIBRARY_PATH=$NRS_HOME/lib:/opt/trend/imss/lib
else
	LD_LIBRARY_PATH=$NRS_HOME/lib
fi

IMSS_DB_DIR=postgresql
ODBCINI=$NRS_HOME/odbc.ini
ODBCSYSINI=$NRS_HOME

export LD_LIBRARY_PATH
export IMSS_DB_DIR
export ODBCINI
export ODBCSYSINI


EXE_NAME=nrslog_parser
EXECUTE=$NRS_HOME/$EXE_NAME
PID_FILE=$NRS_HOME/$EXE_NAME.pid

if [ -f /etc/trend-release ]; then
	trap "" SIGINT
	trap "unlock" EXIT
	
	. /etc/lock_functions
	
	lock
fi

case "$1" in
'start')
     for i in `$PS_CMD | grep -w $EXE_NAME | grep -v grep | grep -v defunct | awk '{ print $2 }'`
     do 
          if test "$i"
          then 
             echo "$EXE_NAME is running."
             exit 1
          fi
     done 
     if test -x $EXECUTE ; then
       $EXECUTE &
       for i in `$PS_CMD | grep -w $EXE_NAME | grep -v grep | awk '{ print $2 }'`
       do 
          if test "$i"
          then 
             echo "$EXE_NAME has started."
             exit 1
          fi
       done 
       echo "$EXE_NAME cannot be started."
     else
       echo "$EXE_NAME is not executable."
       exit 1
     fi
     ;;


'stop')
    if [ -f $PID_FILE ]; then
        PID=`cat $PID_FILE`
        EXE_EXIST=`$PS_CMD | grep $PID | grep -w $EXE_NAME | grep -v grep `
        if test "$EXE_EXIST"
        then
            echo "Shutting down $EXE_NAME $PID ..."
            kill -TERM $PID

                COUNT=0
                while $PS_CMD | grep -w $EXE_NAME | grep -v grep > /dev/null 2>&1 ; do
                sleep 1
                COUNT=`expr $COUNT + 1`
                if test "$COUNT" = "30"
                then
                    for i in `$PS_CMD | grep -w $EXE_NAME | grep -v grep |  awk '{ print $2 }'`
                    do
                        echo "Force to kill $i ..."
                        kill -KILL $i
                    done
                    break
                fi
            done  
        else
            echo "$EXE_NAME $PID is not running."
        fi
    else
        echo "$PID_FILE missing"
        for i in `$PS_CMD |grep -w $EXE_NAME|grep -v grep |awk '{print $2}'`
        do
        	kill -KILL $i
        done
    fi
    ;;
    
'restart')
    if [ -f $PID_FILE ]; then
        PID=`cat $PID_FILE`
        EXE_EXIST=`$PS_CMD | grep $PID | grep -w $EXE_NAME | grep -v grep `
        if test "$IMSS_EXIST"
        then
            echo "Shutting down $EXE_NAME $PID ..."
            kill -TERM $PID

            COUNT=0
            while $PS_CMD | grep -w $EXE_NAME | grep -v grep > /dev/null 2>&1 ; do
                sleep 1
                COUNT=`expr $COUNT + 1`
                if test "$COUNT" = "30"
                then
                    for i in `$PS_CMD | grep -w $EXE_NAME | grep -v grep |  awk '{ print $2 }'`
                    do
                        echo "Force to kill $i ..."
                        kill -KILL $i
                    done
                    break
                fi
            done  
        else
            echo "$EXE_NAME daemon $PID is not running."
            
        fi
    else
        echo "$PID_FILE missing"
        for i in `$PS_CMD |grep -w $EXE_NAME|grep -v grep |awk '{print $2}'`
        do
        	kill -KILL $i
        done
    fi
    if test -x $EXECUTE ; then
       $EXECUTE &
       for i in `$PS_CMD | grep -w $EXE_NAME | grep -v grep | awk '{ print $2 }'`
       do 
          if test "$i"
          then 
             echo "$EXE_NAME has started."
	     exit 1
          fi
       done 
       echo "$EXE_NAME cannot be started."
    else
       echo "$EXECUTE is not executable."
       exit
    fi
       ;;
       
'?')
    EXE_EXIST=`$PS_CMD | grep -w $EXE_NAME | grep -v grep `
    if test "$EXE_EXIST"
    then
        echo "yes"
    else
        echo "no"
    fi
    
    ;;	     	        
    
*)
    echo "usage: $0 start|stop|restart"
    exit 1
    ;;
esac
