#!/bin/sh

# Script for IMSA patch rollbacking via Web Console.
# ini_util.sh is based on StarGate implementation, with minor bug fixes.
# Zhennan, Nov/07/2006
#
# This script is invoked by imssmgr, which is running as root
# when invoked, stdout and stderr will be redirected to log file: patchlog.txt
# for easy testing, no stdout and stderr redirection inside the script.
# this script can be executed from console directly, with correct folder structure
# 
# Naming convention
# F_  means a subroutine

PATCH_FOLDER=${IMSS_HOME}/patch
PATCHMGMINI=${PATCH_FOLDER}/patch.ini
LOCK_FILE=${PATCH_FOLDER}/.isinprogress

NORMAL_ROLLBACK_SH=${PATCH_FOLDER}/rollback.sh
NORMAL_INI_UTIL_SH=${PATCH_FOLDER}/ini_util.sh
NORMAL_PATCH_COMMON_SH=${PATCH_FOLDER}/patch_common.sh
NORMAL_BACKUP_TAR_GZ=${PATCH_FOLDER}/backup.tar.gz
NORMAL_BACKUP_FILELIST=${PATCH_FOLDER}/backup.filelist

#IMSA SP1

NORMAL_APPLY_DB_SH=${PATCH_FOLDER}/apply_db.sh

#End of IMSA SP1

. ${NORMAL_INI_UTIL_SH}
. ${NORMAL_PATCH_COMMON_SH}

PATCH_NUMBER=""
ERROR_NUMBER=0
NO_BACKUP_FILE=0


HaveMoreThanOneVirusFilter()
{
	VIRUS_FILTER_NUMBER=0;
	tmp=`echo "select count(*) from tb_rule where has_virus_filter=1;"|/opt/trend/imss/PostgreSQL/bin/psql -U sa -d imss|sed -n -e '3p'`	
#	if [ "$tmp" -eq 0 ];then
#		VIRUS_FILTER_NUMBER=1
#	fi
  if [ $? -eq 0 ];then
	  VIRUS_FILTER_NUMBER="$tmp"
  fi
}

VirusFilterIsValid()
{
	VIRUS_FILTER_VALID=1;
	tmp=`echo "select rule_value from tb_rule where has_virus_filter=1;"|/opt/trend/imss/PostgreSQL/bin/psql -U sa -d imss|sed -n -e '3p'`	
	SENDER=`echo $tmp | awk -F";" '{print $5}'`
	RECIPIENT=`echo $tmp | awk -F";" '{print $11}'`
	
	if [ "$SENDER" != '*@*' -o "$RECIPIENT" != '*@*' ];then
		VIRUS_FILTER_VALID=0
	fi
	
}

F_check_all() {
    if [ ! -f "$PATCHMGMINI" ] ; then
        F_log "Can not find $PATCHMGMINI."
        ERROR_NUMBER=${ERR_INI_FILE_ACCESS}
        return 1
    fi
    
    if [ ! -f "$NORMAL_BACKUP_FILELIST" ] ; then
        F_log "Can not find $NORMAL_BACKUP_FILELIST."
        ERROR_NUMBER=${ERR_FILE_MISSING}
        return 1
    fi

    # key, section, are required
    
    PATCH_NUMBER=`F_ini_get $PATCHMGMINI Patches Rollbackable`
    if [ -z "$PATCH_NUMBER" ] ; then
        F_log "Cannot find rollbackable patch Number in $PATCHMGMINI."
        ERROR_NUMBER=${ERR_INI_FILE_CONTENT}    
        return 1
    fi
    
    #stop if the patch number can not be found in Applied
    _applied_patches=`F_ini_get $PATCHMGMINI Patches Applied`
    _found_in_applied=""
    for i in $_applied_patches 
    do
        if [ "$PATCH_NUMBER" = "$i" ] ; then
            _found_in_applied="found"
        fi
    done
    if [ -z ${_found_in_applied} ] ; then
        F_log "Cannot find rollbackable patch Number in Applied patches."
        ERROR_NUMBER=${ERR_INI_FILE_CONTENT}    
        return 1
    fi

    F_ini_exist_section ${PATCHMGMINI} "$PATCH_NUMBER"
    if [ $? -ne 0 ] ; then
        F_log "Cannot find rollback info for patch $PATCH_NUMBER."
        ERROR_NUMBER=${ERR_INI_FILE_CONTENT}    
        return 1
    fi

    NO_BACKUP_FILE=`F_ini_get $PATCHMGMINI "$PATCH_NUMBER" "NoBackupFile"`
    if [ $NO_BACKUP_FILE -eq 0 ] ; then
        if [ ! -f "$NORMAL_BACKUP_TAR_GZ" ] ; then
            F_log "Can not find $NORMAL_BACKUP_TAR_GZ."
            ERROR_NUMBER=${ERR_FILE_MISSING}
            return 1
        fi
    fi
   
    IS_MASTER=0
    sh /opt/trend/imss/script/is_master.sh
    if [ $? = 1 ] ; then
        IS_MASTER=1
    fi
 
    if [ $IS_MASTER = 1 ] ; then
    
    	HaveMoreThanOneVirusFilter
    	if [ $VIRUS_FILTER_NUMBER -ne 1 ];then
	    	F_log "None or multi-Virus rules exist."
       		return 1
    	fi
        	
    	VirusFilterIsValid
    	if [ $VIRUS_FILTER_VALID -ne 1 ];then
	    	F_log "Invalid global virus rule: sender or recipient not equal to '*@*'."
	    	return 1
    	fi    
    fi

 # all keys in the $PATCH_NUMBER section are not required.
    F_log "Required file and fields check OK."       	
    return 0
}

F_save_hwmonitor() {

# mount hidden partition
    mkdir ${PATCH_FOLDER}/hiddenOS
    mount /dev/hdc2 ${PATCH_FOLDER}/hiddenOS > /dev/null 2>&1
    if [ $? -ne 0 ] ; then
        mount /dev/hdc3 ${PATCH_FOLDER}/hiddenOS > /dev/null 2>&1
        if [ $? -ne 0 ] ; then
            F_log "Cannot mount image partition, save hardware monitor patch failed."
            return 1
        else
            cp -f /opt/TrendMicro/GoldenGate/bin/hwmonitor ${PATCH_FOLDER}/hiddenOS/opt/TrendMicro/GoldenGate/bin/
            rm -f ${PATCH_FOLDER}/hiddenOS/etc/hwmonmsg.conf.default
            cp -f /etc/init.d/rcS	${PATCH_FOLDER}/hiddenOS/etc/init.d/
            umount /dev/hdc3
        fi
    else
    	cp -f /opt/TrendMicro/GoldenGate/bin/hwmonitor ${PATCH_FOLDER}/hiddenOS/opt/TrendMicro/GoldenGate/bin/
        rm -f ${PATCH_FOLDER}/hiddenOS/etc/hwmonmsg.conf.default
        cp -f /etc/init.d/rcS	${PATCH_FOLDER}/hiddenOS/etc/init.d/
        umount /dev/hdc2
    fi
    rmdir ${PATCH_FOLDER}/hiddenOS
    F_log "Hardware monitor restore successfully."
    return 0
}


F_rollback() {

    IS_MASTER=0
    sh /opt/trend/imss/script/is_master.sh
    if [ $? = 1 ] ; then
        IS_MASTER=1
    fi

    _value=`F_ini_get ${PATCHMGMINI} ${PATCH_NUMBER} "UIInterruption"`
    if [ ! -z "$_value" ] ; then
    if [ $_value = "1" ] ; then
    if [ $IS_MASTER = 1 ] ; then
        /opt/trend/imss/PostgreSQL/bin/psql -U sa -d imss -c "update tb_patch_status set status=4 where status=2;" > /dev/null 2>&1
        sleep 10
        /opt/trend/imss/PostgreSQL/bin/psql -U sa -d imss -c "update tb_patch_status set status=2 where status=4;" > /dev/null 2>&1
    fi
    fi
    fi
    
    _value=`F_ini_get ${PATCHMGMINI} ${PATCH_NUMBER} "CmdBeforeUpdate"`
    if [ ! -z "$_value" ] ; then
        F_log "Calling ${_value}"
        sh -c "${_value}"
    fi
    
    
#IMSA SP1
#Need remount it to read-write
mount -o remount,rw /
mv /opt/TrendMicro/GoldenGate/bin/hwmonitor /opt/TrendMicro/GoldenGate/bin/hwmonitor.bak

#End of IMSA SP1
    
    # no need to check return value of "rm -rf"
    # if it's failed in the middle, better keep rollbacking...
    pushd .
    cd /
    for i in `cat ${NORMAL_BACKUP_FILELIST}`
    do 
    	rm -rf $i
    done
    popd
    
    if [ $NO_BACKUP_FILE -eq 0 ] ; then
        tar -C / -zxf ${NORMAL_BACKUP_TAR_GZ}
        if [ $? -ne 0 ] ; then
            F_log "Cannot untar files."
            ERROR_NUMBER=${ERR_UNTAR_BACKUP_FILES}
            
            # although this probably won't work at this time...
            _value=`F_ini_get ${PATCHMGMINI} ${PATCH_NUMBER} "CmdAfterUpdate"`
            if [ ! -z "$_value" ] ; then
                F_log "Calling ${_value}"
                sh -c "${_value}"
            fi
            return 1
        fi  
    fi
 
 
#IMSA SP1

F_save_hwmonitor

#remove the bak file, restart hardware monitor
rm -f /opt/TrendMicro/GoldenGate/bin/hwmonitor.bak
chmod u+x /opt/TrendMicro/GoldenGate/bin/hwmonitor 
kill `ps -ef | grep 'opt/TrendMicro/GoldenGate/bin/hwmonitor' |grep -v grep | awk '{if($3==1)print $2}'`

#Restore it back to read-only
mount -o remount,ro /

restore_agentbuild

#remove UI's temporary file, force to generate new cache
rm -rf /opt/trend/imss/UI/adminUI/ROOT/work
rm -rf /opt/trend/imss/UI/euqUI/ROOT/work

#End of IMSA SP1
   

    # must write result before call CmdAfterUpdate
    # otherwise, imssmgr may not able to find the result during startup
    # for rollback, result_build and patch_title are useless
    F_write_result "1" "2" "0" "0" ${ERR_MSG_SUCCESS} "result_build" "patch_title"
    
    _value=`F_ini_get ${PATCHMGMINI} ${PATCH_NUMBER} "CmdAfterUpdate"`
    if [ ! -z "$_value" ] ; then
        F_log "Calling ${_value}"
        sh -c "${_value}"
    fi
    
    # update patch.ini
    
    F_ini_delete_section ${PATCHMGMINI} ${PATCH_NUMBER}
    F_ini_delete_key ${PATCHMGMINI} "Patches" "Rollbackable"
    _applied_patches=`F_ini_get $PATCHMGMINI "Patches" "Applied"`
    _new_applied_patches=" "
    
    for i in $_applied_patches 
    do
        if [ "$PATCH_NUMBER" != "$i" ] ; then
            _new_applied_patches="${_new_applied_patches} $i"
        fi
    done
    
    F_ini_set $PATCHMGMINI "Patches" "Applied" "${_new_applied_patches}"
    
    # remove backup files
    
    rm -f ${NORMAL_ROLLBACK_SH}
    rm -f ${NORMAL_INI_UTIL_SH}
    rm -f ${NORMAL_PATCH_COMMON_SH}
    rm -f ${NORMAL_BACKUP_TAR_GZ}
    rm -f ${NORMAL_BACKUP_FILELIST}
    
    return 0
}

restore_agentbuild()
{
	if [ -s ${IMSS_HOME}/config/Agent.ini ];then
		sed -e "s,Batch_Queue_Size=10000000,Batch_Queue_Size=500000,g" ${IMSS_HOME}/config/Agent.ini >${PATCH_FOLDER}/pp$$
		cp ${PATCH_FOLDER}/pp$$ ${IMSS_HOME}/config/Agent.ini
		rm -f ${PATCH_FOLDER}/pp$$	
		
		OFFHOUR_STATUSLOG=`grep 'Offhour_Statuslog=' ${IMSS_HOME}/config/Agent.ini`
		OFFHOUR_VIRUSLOG=`grep 'Offhour_VirusLog=' ${IMSS_HOME}/config/Agent.ini`
		
		F_log "1" "$OFFHOUR_STATUSLOG"
		F_log "1" "$OFFHOUR_VIRUSLOG"
		
		ed ${IMSS_HOME}/config/Agent.ini <<EOF > /dev/null 2>&1
/Offhour_Statuslog
.
d
.
/Offhour_VirusLog
.
d
.
/Offhour_SecurityLog
.
d
.
/\[Common\]
.+1
i
$OFFHOUR_STATUSLOG
$OFFHOUR_VIRUSLOG
.
w
q
EOF
    fi
}

F_main() {

    F_log "Start rollback patch."
    
    if [ -f "$LOCK_FILE" ] ; then
        F_log "Another installing/uninstalling is in progress."
        ERROR_NUMBER=${ERR_ISINPROGRESS}
        return 1
    fi

    touch ${LOCK_FILE}
    
    if [ $? -ne 0 ] ; then
        F_log "Can not create lock file."
        ERROR_NUMBER=${ERR_FAILED_TO_LOCK}
        return 1
    fi
    
    # use size to indicate type, zero is update, one is rollback
    # this is used in IMSA 1, although IMSA 7 doesn't need this, just keep it.
    echo "r" > ${LOCK_FILE}
    
    F_check_all
    
    if [ $? -ne 0 ] ; then
        rm -f ${LOCK_FILE}
        return 1
    fi
        
    F_rollback

    if [ $? -ne 0 ] ; then
        rm -f ${LOCK_FILE}
        return 1
    fi
    
    # Success
    F_log "Done. Success"
    rm -f ${LOCK_FILE}

    # remove the package in hidden partition. Ignore error here.
    mount /dev/sda6 /mnt/backup > /dev/null 2>&1
    if [ $? -ne 0 ] ; then
        umount /dev/sda6 > /dev/null 2>&1
        mount /dev/sda6 /mnt/backup > /dev/null 2>&1
        if [ $? -ne 0 ] ; then
            F_log "Cannot mount image partition, remove application patch failed."
            return 0
        fi
    fi

    rm -f /mnt/backup/patches/${PATCH_NUMBER} > /dev/null 2>&1

    cat /mnt/backup/patches/patches.history | grep -v "^${PATCH_NUMBER}$" > /mnt/backup/patches/patches.history.new
    rm -f /mnt/backup/patches/patches.history
    mv /mnt/backup/patches/patches.history.new /mnt/backup/patches/patches.history

    umount /dev/sda6

    return 0
}

F_main
if [ $? -ne 0 ] ; then

    # for rollback, result_build and patch_title are useless
    F_write_result "1" "2" "0" "1" "${ERR_MSG_FAIL}" "result_build" "patch_title"
    exit ${ERROR_NUMBER}
else
    exit 0
fi
